package dk.firma.klient.webservice;

import dk.gov.oiosi.common.schedule.ScheduledExecutor;
import dk.gov.oiosi.configuration.ConfigurationException;
import dk.gov.oiosi.configuration.ConfigurationHandler;
import dk.gov.oiosi.security.CertificateSubject;
import dk.gov.oiosi.security.ldap.LdapCertificateLookup;
import dk.gov.oiosi.security.lookup.CertificateLookupException;
import dk.gov.oiosi.security.revocation.IRevocationLookup;
import dk.gov.oiosi.security.revocation.RevocationException;
import dk.gov.oiosi.security.revocation.RevocationLookupFactory;
import dk.gov.oiosi.security.revocation.RevocationResponse;
import dk.gov.oiosi.security.validation.CertificateExpiredException;
import dk.gov.oiosi.security.validation.CertificateNotActiveException;
import dk.gov.oiosi.security.validation.CertificateValidator;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.security.cert.X509Certificate;

public class TestCertificate {

    private static Log log = LogFactory.getLog(OiosiRaspClient.class);
    public static final String RaspConfigurationLive = "./Resources/RaspConfiguration.Live.xml";
    public static final String RaspConfigurationTest = "./Resources/RaspConfiguration.Test.xml";

    public TestCertificate() throws Exception {
        // First the certificate is retrieved from LDAP
        // Then the fate/time is validated
        // last the revocation is check in OCSP

        String subjectString;

        // Production configuration
        ConfigurationHandler.setConfigurationFile(RaspConfigurationLive);
        // Production certificates valid
        subjectString = "SERIALNUMBER=CVR:34051178-FID:1359538750995 + CN=DIGST Demo endpoint (funktionscertifikat), O=Digitaliseringsstyrelsen // CVR:34051178, C=DK";
        // Production certificates not valid
        //subjectString = "SERIALNUMBER = CVR:26911745-FID:1200662306791 + CN=KMD OIOSI Gateway 2 (funktionscertifikat), O=KMD A/S // CVR:26911745, C=DK";
        // Expired -> // subjectString = "CN = NemHandel test service (funktionscertifikat) + SERIALNUMBER = CVR:26769388-FID:1200406941690, O = IT- og Telestyrelsen // CVR:26769388, C = DK";

        // Test configuration
        //ConfigurationHandler.setConfigurationFile(RaspConfigurationTest);
        // Test certificates valid
        //subjectString = "CN = Testendpoint (funktionscertifikat) + SERIALNUMBER = CVR:26769388-FID:1208430425605, O = IT- og Telestyrelsen // CVR:26769388, C = DK";
        // subjectString = "CN = Testendpoint (funktionscertifikat) + SERIALNUMBER = CVR:26769388-FID:1208430425605, O = IT- og Telestyrelsen // CVR:26769388, C = DK";
        // subjectString = "CN = TEST FOCES1 (funktionscertifikat) + SERIALNUMBER = CVR:30808460-FID:1320135775022, O = DANID A/S // CVR:30808460, C = DK";
        // Test certificates not valid
        //subjectString = "CN = TU GENEREL FOCES gyldig (funktionscertifikat) + SERIALNUMBER = CVR:30808460-FID:94731315, O = Danid A/S // CVR:30808460, C = DK";

        // subjectString = "CN = Test NemHandelservice  (funktionscertifikat) + SERIALNUMBER = CVR:34051178-FID:1329398684312, O = Digitaliseringsstyrelsen // CVR:34051178, C = DK";

        // not valid any more
        // subjectString = "CN=TU GENEREL FOCES gyldig (funktionscertifikat) + SERIALNUMBER=CVR:30808460-FID:94731315, O=Danid A/S // CVR:30808460, C=DK";
        // subjectString = "CN=FOCES1 (funktionscertifikat) + SERIALNUMBER=CVR:30808460-FID:1255692730737, O=DANID A/S // CVR:30808460, C=DK";
        //subjectString = "CN=TU GENEREL MOCES gyldig + SERIALNUMBER=CVR:30808460-RID:45490598, O=Danid A/S // CVR:30808460, C=DK";

        // Use either production Rasp or test Rasp

        X509Certificate certificate = this.retrieveCertificateFromLdap(subjectString);
        if (certificate != null) {
            // The certificate is public, and can be used.
            boolean publicCertificate = this.validateCertificate(certificate);

            if (publicCertificate) {
                // Validate if the certificate is Revocation
                IRevocationLookup lookup = RevocationLookupFactory.createRevocationLookupClient();

                RevocationResponse response = this.validateCertificateInOCSP(lookup, certificate);
                if (response.isValid()) {
                    // already printed out
                    //log.info("Certificate valid in OCSP");
                } else {
                    // already printed out
                    //log.warn("Certificate NOT valid in OCSP");
                }
            } else {
                log.info("certificate could not be retrieved - it is not public.");
            }
        }
    }


    public X509Certificate retrieveCertificateFromLdap(String subjectString) throws ConfigurationException, RevocationException {
        // We can not test a subject string, so retrieving the certificate from
        LdapCertificateLookup lookup = new LdapCertificateLookup();
        X509Certificate certificate = null;

        try {
            CertificateSubject certificateSubject = new CertificateSubject(subjectString);
            certificate = lookup.getCertificate(certificateSubject);
            log.info("Certificate to validate retrieved");
        } catch (CertificateNotActiveException e) {
            log.error("Certificate not yet active!", e);
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        } catch (CertificateExpiredException e) {
            log.error("Certificate expired!", e);
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        } catch (CertificateLookupException e) {
            log.error("Certificate not retrieved!", e);
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }

        return certificate;
    }

    public boolean validateCertificate(X509Certificate certificate) {
        boolean result = false;

        try {
            CertificateValidator.validateCertificate(certificate);
            result = true;
        } catch (CertificateNotActiveException e) {
            log.info("Certificate not valid - not yet active");
        } catch (CertificateExpiredException e) {
            log.info("Certificate not valid - past expired date");
        }

        return result;
    }

    public RevocationResponse validateCertificateInOCSP(IRevocationLookup lookup, X509Certificate x509Certificate) throws RevocationException {
        CertificateSubject subject = new CertificateSubject(x509Certificate);
        log.info("Getting the RevocationResponse for: " + subject);
        RevocationResponse revocationResponse = lookup.checkCertificate(x509Certificate);

        if (revocationResponse.isValid()) {
            log.info("Certificate is valid (not revoked)");
        } else {
            log.info("Certificate IS revoked!! No longer valid");
        }

        return revocationResponse;
    }

    public static void main(String[] args) {
        try {
            log.info("Start validate the certificate based on the subject");
            new TestCertificate();
            log.info("Finish");
            ScheduledExecutor.getInstance().shutdown();
        } catch (RevocationException e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}