package dk.firma.klient.webservice;

import dk.gov.oiosi.addressing.Identifier;
import dk.gov.oiosi.addressing.IdentifierEan;
import dk.gov.oiosi.common.Utilities;
import dk.gov.oiosi.common.schedule.ScheduledExecutor;
import dk.gov.oiosi.communication.Uri;
import dk.gov.oiosi.configuration.ConfigurationHandler;
import dk.gov.oiosi.exception.MainException;
import dk.gov.oiosi.uddi.*;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.List;

public class TestUddiLookup {

    private static Log log = LogFactory.getLog(OiosiRaspClient.class);

    public TestUddiLookup() {
    }

    public void perform() throws Exception {
        // Environment
        ConfigurationHandler.setConfigurationFile("Resources/RaspConfiguration.Live.xml"); // Production
        //ConfigurationHandler.setConfigurationFile("cfg/RaspConfiguration.Test.xml"); // Test

        // endpoint
        Identifier identifier;
        identifier = new IdentifierEan("5798009811578");
        //identifier = new IdentifierEan("5798000418554");
        // identifier = new IdentifierCvr("25615468");


        // tModel key for document type
        // ToDo - Get the tModel key, base on the root namespace of the document
        String serviceContractTModelOioublInvoice = "uddi:2e0b402a-7a5e-476b-8686-b33f54fd1f47"; // OIOUBL Invoice

        UddiId serviceId = new UddiGuidId(serviceContractTModelOioublInvoice, true);

        // tModel key for the Profile
        UddiId profileId;

        //profileId = Utilities.getProfileId("OIOXML elektronisk handel");
        //profileId = Utilities.getProfileId("OIOXML elektronisk handel - læs ind");
        //profileId = Utilities.getProfileId("OIOXML Elektronisk Regning");
        //profileId = Utilities.getProfileId("OIOXML Elektronisk Kreditnota");
        //profileId = Utilities.getProfileId("Catalogue-CatBas-1.0");
        //profileId = Utilities.getProfileId("Catalogue-CatBasR-1.0");
        //profileId = Utilities.getProfileId("Catalogue-CatSim-1.0");
        //profileId = Utilities.getProfileId("Catalogue-CatSimR-1.0");
        //profileId = Utilities.getProfileId("Catalogue-CatExt-1.0");
        //profileId = Utilities.getProfileId("Catalogue-CatExtR-1.0");
        //profileId = Utilities.getProfileId("Catalogue-CatAdv-1.0");
        //profileId = Utilities.getProfileId("Catalogue-CatAdvR-1.0");
        profileId = Utilities.getProfileId("Procurement-BilSim-1.0");
        //profileId = Utilities.getProfileId("Procurement-BilSimR-1.0");
        //profileId = Utilities.getProfileId("Procurement-PayBas-1.0");
        //profileId = Utilities.getProfileId("Procurement-PayBasR-1.0");
        //profileId = Utilities.getProfileId("Procurement-OrdSim-BilSim-1.0");
        //profileId = Utilities.getProfileId("Procurement-OrdSimR-BilSimR-1.0");
        //profileId = Utilities.getProfileId("Procurement-OrdSim-BilSimR-1.0");
        //profileId = Utilities.getProfileId("Procurement-OrdSimR-BilSim-1.0");
        //profileId = Utilities.getProfileId("Procurement-OrdAdv-BilSim-1.0");
        //profileId = Utilities.getProfileId("Procurement-OrdAdv-BilSimR-1.0<");
        //profileId = Utilities.getProfileId("Procurement-OrdAdvR-BilSim-1.0");
        //profileId = Utilities.getProfileId("Procurement-OrdAdvR-BilSimR-1.0");
        //profileId = Utilities.getProfileId("Procurement-OrdSel-BilSim-1.0");
        //profileId = Utilities.getProfileId("Procurement-OrdSel-BilSimR-1.0");
        //profileId = Utilities.getProfileId("urn:www.nesubl.eu:profiles:profile1:ver2.0");
        //profileId = Utilities.getProfileId("urn:www.nesubl.eu:profiles:profile2:ver2.0");
        //profileId = Utilities.getProfileId("urn:www.nesubl.eu:profiles:profile3:ver2.0");
        //profileId = Utilities.getProfileId("urn:www.nesubl.eu:profiles:profile4:ver2.0");
        //profileId = Utilities.getProfileId("urn:www.nesubl.eu:profiles:profile5:ver2.0");
        //profileId = Utilities.getProfileId("urn:www.nesubl.eu:profiles:profile6:ver2.0");
        //profileId = Utilities.getProfileId("urn:www.nesubl.eu:profiles:profile7:ver2.0");
        //profileId = Utilities.getProfileId("urn:www.nesubl.eu:profiles:profile8:ver2.0");
        //profileId = Utilities.getProfileId("Reference-Utility-1.0");
        //profileId = Utilities.getProfileId("Reference-UtilityR-1.0");


        List<UddiId> profileDefinitionIds = new ArrayList<UddiId>();
        profileDefinitionIds.add(profileId);

        // Accepted EndpointType
        List<EndpointAddressTypeCode> acceptedTransportProtocols = new ArrayList<EndpointAddressTypeCode>();
        acceptedTransportProtocols.add(EndpointAddressTypeCode.http);


        LookupParameters lookupParameters = new LookupParameters(
                identifier,
                serviceId,
                profileDefinitionIds,
                acceptedTransportProtocols
        );

        UddiLookupClient uddiClient = this.createOriginalUddiLookupClient();
        log.info("Lookup clients created.");

        List<UddiLookupResponse> uddiLookupResponseList = uddiClient.lookup(lookupParameters);
        log.info("Lookup returned " + uddiLookupResponseList.size() + " uddiLookupResponse.");

        for (UddiLookupResponse uddiLookupResponse : uddiLookupResponseList) {
            // ToDo - Print out the endpoints
            log.info(uddiLookupResponse);
        }

        log.info("Finish");
    }


    public UddiLookupClient createOriginalUddiLookupClient() throws MainException, URISyntaxException {
        String url = "http://discoverypublic.uddi.ehandel.gov.dk/registry/uddi/inquiry";
        return createLookupClient(url);
    }

    public UddiLookupClient createLookupClient(String url) throws URISyntaxException, MainException {
        Uri uri = new Uri(url);
        return new UddiLookupClient(uri);
    }

    public static void main(final String[] args) {
        try {
            new TestUddiLookup().perform();
            ScheduledExecutor.getInstance().shutdown();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}