/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.axis2.module;

import dk.gov.oiosi.configuration.ConfigurationHandler;
import dk.gov.oiosi.policy.OiosiPolicyBean;
import org.apache.axiom.om.OMElement;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.engine.Handler;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.neethi.Assertion;
import org.apache.neethi.Policy;
import org.apache.neethi.builders.xml.XmlPrimtiveAssertion;

import java.util.List;

/**
 * Base for handles.
 */
public abstract class RaspHandler implements Handler {

    private Log log = LogFactory.getLog(RaspHandler.class);

    /**
     * @deprecated See {@link Handler}.cleanup method
     */
    public void cleanup() {
    }

    /**
     * Utility to extract policy/configuration bean.
     *
     * @param msgContext message context
     * @return policy bean
     */
    protected OiosiPolicyBean getPolicyBean(MessageContext msgContext) {
        log.debug("OIOSIHandler.getPolicyBean: " + msgContext.getAxisService().getName());

        OiosiPolicyBean oiosiPolicyBean;

        // Check if present in properties
        if (msgContext.getProperty(RaspModule.OIOSI_PROPERTY_BEAN) != null) {
            oiosiPolicyBean = (OiosiPolicyBean) msgContext.getProperty(RaspModule.OIOSI_PROPERTY_BEAN);
            log.debug("Use transferred Policy");
            return oiosiPolicyBean;
        }

        // Create default Values
        oiosiPolicyBean = new OiosiPolicyBean();

        // Try to init from service configuration
        try {
            Policy servicePolicy = msgContext.getEffectivePolicy();
            log.debug("Found service policy");
            if (servicePolicy == null || servicePolicy.getAlternatives() == null) {
                return oiosiPolicyBean;
            }

            List<?> it = (List<?>) servicePolicy.getAlternatives().next();
            for (Object anIt : it) {
                Assertion assertion = (Assertion) anIt;

                if (OiosiPolicyBean.RASP_POLICY_NAMESPACE.equals(assertion.getName().getNamespaceURI())
                        && OiosiPolicyBean.RASP_POLICY_BEAN.equals(assertion.getName().getLocalPart())) {

                    XmlPrimtiveAssertion xpa = (XmlPrimtiveAssertion) assertion;
                    OMElement assertionValue = xpa.getValue();

                    oiosiPolicyBean.moveValues(assertionValue);

                    if (oiosiPolicyBean.getRaspConfiguration() != null) {
                        log.debug("Setting Custom RaspConfiguration.xml: " + oiosiPolicyBean.getRaspConfiguration());
                        ConfigurationHandler.setConfigurationFile(oiosiPolicyBean.getRaspConfiguration());
                    }
                    break;
                }
            }
        } catch (Exception e) {
            log.error("Failed to initialize OIOSIPolicyBean - use default values: " + e.getMessage(), e);
        }

        // Save for this message context...
        msgContext.setProperty(RaspModule.OIOSI_PROPERTY_BEAN, oiosiPolicyBean);

        return oiosiPolicyBean;
    }
}