/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.axis2.module;

import java.util.Hashtable;
import java.util.Map;
import org.apache.axis2.AxisFault;
import org.apache.axis2.context.ConfigurationContext;
import org.apache.axis2.description.AxisDescription;
import org.apache.axis2.description.AxisModule;
import org.apache.axis2.modules.Module;
import org.apache.neethi.Assertion;
import org.apache.neethi.Policy;

/**
 * RASP Module.
 */
public class RaspModule implements Module {

    /**
     * module name.
     */
    public static final String MODULE_NAME = "rasp";

    /**
     * property name for holding policybean in message context.
     */
    public static final String OIOSI_PROPERTY_BEAN = "OIOSI_PROPERTY_BEAN";

    /**
     * instance.
     */
    private static RaspModule instance = null;

    /**
     * namespace url used for RM.
     */
    Map<String, String> rmNamespaces = new Hashtable<String, String>();


    /**
     * Util to test for RM message.
     * 
     * @param namespace
     *            namespace url
     * @return rm message
     */
    private boolean isRMNamespace(String namespace) {
        if (rmNamespaces.get(namespace) != null) {
            return true;
        }
        return false;
    }

    /**
     * Util to test for RM message.
     * 
     * @param namespace
     *            namespace url
     * @return rm message
     */
    public boolean isRMNamespaceByAction(String namespace) {
        if (namespace == null) {
            return false;
        }
        int ix = namespace.lastIndexOf("/");
        if (ix > 0) {
            String test = namespace.substring(0, ix);
            return isRMNamespace(test);
        } else {
            // NO URL ==> NOT RM
            return false;
        }
    }

    /**
     * Constructor.
     */
    public RaspModule() {
        instance = this;
    }

    /**
     * Return instance.
     * 
     * @return rasp module instance
     */
    public static RaspModule getInstance() {
        return instance;
    }

    private static final String SPEC_2005_02_NS_URI = "http://schemas.xmlsoap.org/ws/2005/02/rm";

    private static final String SPEC_2007_02_NS_URI = "http://docs.oasis-open.org/ws-rx/wsrm/200702";

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.axis2.modules.Module#init(org.apache.axis2.context.ConfigurationContext,
     *      org.apache.axis2.description.AxisModule)
     */
    public void init(ConfigurationContext configContext, AxisModule module) throws AxisFault {
        rmNamespaces.put(SPEC_2005_02_NS_URI, SPEC_2005_02_NS_URI);
        rmNamespaces.put(SPEC_2007_02_NS_URI, SPEC_2007_02_NS_URI);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.axis2.modules.Module#engageNotify(org.apache.axis2.description.AxisDescription)
     */
    public void engageNotify(AxisDescription axisDescription) throws AxisFault {
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.axis2.modules.Module#shutdown(org.apache.axis2.context.ConfigurationContext)
     */
    public void shutdown(ConfigurationContext configurationContext) throws AxisFault {
        // at the moment, nothing needs to be done ..
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.axis2.modules.Module#applyPolicy(org.apache.neethi.Policy, org.apache.axis2.description.AxisDescription)
     */
    public void applyPolicy(Policy arg0, AxisDescription arg1) throws AxisFault {
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.axis2.modules.Module#canSupportAssertion(org.apache.neethi.Assertion)
     */
    public boolean canSupportAssertion(Assertion arg0) {
        return true;
    }

}
