/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.common;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.TimeZone;

/**
 * Format date for UDDI.
 *
 * Handles both new Format and deprecated format.
 */
public final class DateUtil {

    /**
     * Hide constructor.
     */
    private DateUtil() {
    }

    /**
     * New format in UDDI : "2019-09-07T00:00:00.0000000Z".
     */
    private static SimpleDateFormat formatNew;

    /**
     * Old depricated format.
     */
    private static SimpleDateFormat formatOld;

    static {
        formatNew = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss z");
        formatNew.setTimeZone(TimeZone.getTimeZone("UTC"));

        formatOld = new SimpleDateFormat("yyyy-MM-dd");
        formatOld.setTimeZone(TimeZone.getTimeZone("UTC"));
    }

    /**
     * Parse date from UDDI.
     *
     * @param date
     *            date from UDDI
     * @return date or null if illegal
     */
    public static Date parseDate(final String date) {
        try {
            return formatNew.parse(date);
        } catch (Exception ignore) {
            // ignore
        }
        try {
            return formatOld.parse(date);
        } catch (Exception ignore) {
            // ignore
        }
        return null;
    }

    /**
     * Format a date in UDDI format.
     *
     * @param d
     *            date to format
     * @return the formatted date
     */
    public static String formatDate(final Date d) {
        return formatNew.format(d);
    }
}
