/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.common.cache;

import dk.gov.oiosi.common.schedule.AbstractScheduleJob;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Schedule job to check expiration of ICache implementations, used for TimedCache
 *
 * @param <TKey>
 * @param <TValue>
 * @since OIORASP 1.3.0
 */
public class CacheExpiredCheckTask<TKey, TValue> extends AbstractScheduleJob {

    private static final long serialVersionUID = -6535996895146041968L;
    private static final Log log = LogFactory.getLog(CacheExpiredCheckTask.class);

    private final ICache<TKey, TValue> cache;

    public CacheExpiredCheckTask(ICache<TKey, TValue> cache) {
        this.cache = cache;
    }

    private String buildJobDetails() {
        StringBuilder sb = new StringBuilder();
        sb.append("CacheExpiredCheck for cache ");
        sb.append(this.cache);
        return sb.toString();
    }

    @Override
    public boolean doJob() {
        long start = System.currentTimeMillis();
        int expiredCount = this.cache.checkExpired();
        if (log.isDebugEnabled()) {
            log.debug("Checked cache for expiration, done in " + (System.currentTimeMillis() - start) + " ms, expired " + expiredCount + " values, cache " + this.cache);
        }
        return true;
    }

    @Override
    public String getJobDetails() {
        return buildJobDetails();
    }

    @Override
    public Log getLog() {
        return log;
    }
}