package dk.gov.oiosi.common.cache;

import dk.gov.oiosi.common.cache.holder.IMultipleQuantityValueHolder;
import dk.gov.oiosi.common.cache.holder.SimpleValueHolder;

import java.util.Map;

/**
 * Cache algorithm: Least Recently Used (LRU). "Age bits" (see <a
 * href="http://en.wikipedia.org/wiki/Cache_algorithms">Wiki: Cache_algorithms</a>)
 * are kept by _cacheUsageIndex in parent class. Each time something is returned
 * by tryGetValue method, this value is incremented and ValueHolder is updated
 * with result. Cache value, which has lowest such value, is removed when the
 * cache is full.
 *
 * @author Dmitriy Lapko, TrueLink A/S
 *
 * @param <TKey>
 * @param <TValue>
 *
 * @since OIORASP 1.3.0
 */
public class LeastRecentlyUsedQuantityCache<TKey, TValue> extends MultipleQuantityCache<TKey, TValue> {

	public LeastRecentlyUsedQuantityCache(int maxSize) {
		super(maxSize);
	}

	public LeastRecentlyUsedQuantityCache(Map<String, String> configuration) {
		super(configuration);
	}

	@Override
	protected IMultipleQuantityValueHolder<TKey, TValue> buildValueHolder(TKey key) {
		return new SimpleValueHolder<TKey, TValue>(key);
	}

}
