/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.common.cache;

import java.util.HashMap;
import java.util.Map;

import dk.gov.oiosi.common.OutVariable;
import dk.gov.oiosi.common.cache.stat.AbstractStatisticsCollectorCache;

/**
 * Dummy cache. Caches nothing.
 * 
 * Always returns false.
 */
public class ZeroCache<TKey, TValue> extends AbstractStatisticsCollectorCache<TKey, TValue> implements IThreadSafeCache<TKey, TValue> {

    private final Map<String, String> configuration;

	public ZeroCache()
    { 
		this.configuration = new HashMap<String, String>();
    }

    public ZeroCache(Map<String, String> configuration)
    {
		this.configuration = configuration;
    }

    /**
     * Do nothing.
     * @param key key for cached item
     * @param value item to cache
     */            
    public final void add(final TKey key, final TValue value) { }
    
    public final void set(final TKey key, final TValue value) { }

    /**
     * Always return false.
     * @param key key to search for
     * @return false always
     */
    public final boolean containsKey(final TKey key) {
        return false;
    }

    /**
     * Do nothing.
     * @param key remove item for key
     */
    public final void remove(final TKey key) { }

    /**
     * Always return null.
     * @param key
     *            search for item with key
     * @return always null
     */
    public final boolean tryGetValue(final TKey key, OutVariable<TValue> value) {
        return false;
    }

    /**
     * Do nothing, no data cached
     */
    public int checkExpired()
    {
        // nothing to do - we do not cache anything
    	return 0;
    }

	public Map<String, String> getConfiguration() {
		return configuration;
	}

    @Override
    public int getSize() {
        return 0;  //To change body of implemented methods use File | Settings | File Templates.
    }

    @Override
	public String toString() {
		return "ZeroCache";
	}
}