package dk.gov.oiosi.common.cache.holder;

import dk.gov.oiosi.common.cache.MultipleQuantityCache;

/**
 * Value holder with one value only for implementation Least frequently used
 * cache policy by {@link MultipleQuantityCache} implementations.
 * 
 * @author Dmitriy Lapko, TrueLink A/S
 * 
 * @param <TKey>
 * @param <TValue>
 */
public class SingleValueHolder<TKey, TValue> extends SimpleValueHolder<TKey, TValue> implements IMultipleQuantityValueHolder<TKey, TValue> {

	private volatile int useCount;

	public SingleValueHolder(TKey key) {
		super(key);
	}

	@Override
	public int incrementUseCount(int cacheUsageIndex) {
		return this.useCount++;
	}

	@Override
	public int getUseCount() {
		return this.useCount;
	}

	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder();
		sb.append("used=");
		sb.append(useCount);
		sb.append(", value=");
		sb.append(value);
		return sb.toString();
	}

}
