package dk.gov.oiosi.common.cache.stat;

import dk.gov.oiosi.common.cache.ICache;
import dk.gov.oiosi.common.schedule.AbstractScheduleJob;
import dk.gov.oiosi.configuration.CacheFactory;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Scheduled job, which logs state of all caches. Uses logger, configured to this class, with info level
 *
 * @author Dmitriy Lapko, TrueLink A/S
 * @since OIORASP 1.3.0
 */
public class CacheStateLogScheduledJob extends AbstractScheduleJob {

    public static final Log log = LogFactory.getLog(CacheStateLogScheduledJob.class);

    private static final long serialVersionUID = -1654753751335563966L;

    public CacheStateLogScheduledJob() {
    }

    @Override
    public boolean doJob() {
        if (log.isInfoEnabled()) {
            String cacheListState = buildCacheListStateString();
            log.info(cacheListState);
        }
        return true;
    }

    protected String buildCacheListStateString() {
        StringBuilder sb = new StringBuilder();
        sb.append("Caches state:\n  ");
        sb.append(generateState("CertificateCache", CacheFactory.getInstance().getCertificateCache()));
        sb.append("\n  ");
        sb.append(generateState("CrlLookupCache", CacheFactory.getInstance().getCrlLookupCache()));
        sb.append("\n  ");
        sb.append(generateState("OcspLookupCache", CacheFactory.getInstance().getOcspLookupCache()));
        sb.append("\n  ");
        sb.append(generateState("SchematronStoreCache", CacheFactory.getInstance().getSchematronStoreCache()));
        sb.append("\n  ");
        sb.append(generateState("UddiServiceCache", CacheFactory.getInstance().getUddiServiceCache()));
        sb.append("\n  ");
        sb.append(generateState("UddiTModelCache", CacheFactory.getInstance().getUddiTModelCache()));
        return sb.toString();
    }

    protected String generateState(String cacheName, ICache<?, ?> cache) {
        StringBuilder sb = new StringBuilder();
        sb.append(cacheName);
        sb.append("\n\ttoString():\t");
        sb.append(cache);
        if (cache instanceof AbstractStatisticsCollectorCache<?, ?>) {
            AbstractStatisticsCollectorCache<?, ?> c = (AbstractStatisticsCollectorCache<?, ?>) cache;
            sb.append("\n\t");
            sb.append("Statistics:\t");
            sb.append(c.getStatData());
        }
        return sb.toString();
    }

    @Override
    public String getJobDetails() {
        return "CacheStateLog";
    }

    @Override
    public Log getLog() {
        return log;
    }
}