package dk.gov.oiosi.common.startup;

import dk.gov.oiosi.exception.MainException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.util.Collections;
import java.util.List;

/**
 * Abstract class for startup tasks - guarantees non-null configuration list and
 * provides general logs for all tasks.
 *
 * To enable logging of startup tasks execution details, set log level INFO for
 * logger "startupTasks".
 *
 * @author Dmitriy Lapko, TrueLink A/S
 * @since OIORASP 1.3.0
 */
public abstract class AbstractStartupTask implements IStartupTask {

	/*
	 * Extracted to be accessible from unit tests in different packages. Use
	 * special name startupTasks instead of
	 * dk.gov.oiosi.common.startup.AbstractStartupTask for convenience and more
	 * clear configuration in log4j.xml
	 */
	public static final String LOG_NAME = "startupTasks";

	/*
	 * Separate log configuration for all start up tasks to be able easy turn on
	 * all logging at start and turn it off.
	 *
	 * If some implementation requires separate logging, it should use own log
	 * object.
	 */
	protected static final Log log = LogFactory.getLog(LOG_NAME);

	protected List<StartupTaskConfiguration> configList = Collections.emptyList();

	@Override
	public void init(List<StartupTaskConfiguration> configList) {
		if (configList == null) {
			this.configList = Collections.emptyList();
		} else {
			this.configList = configList;
		}
	}

	public void setConfigList(List<StartupTaskConfiguration> configList) {
		this.init(configList);
	}

	public List<StartupTaskConfiguration> getConfigList() {
		return configList;
	}

	@Override
	public abstract void doTask() throws MainException;


}
