/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.communication;

import dk.gov.oiosi.security.oces.OcesX509Certificate;

/**
 * Representing RASP credentials.
 * 
 * NOTE: In Java/Axis it is only the Server/Receiver party certificate which is
 * specified. Sender/Self certificate/key could in future be specified as a
 * keylabel for Rampart.
 */
public class Credentials {
    
	/**
     * The client/sender certificate.
     */
    private OcesX509Certificate clientCertificate;
    
	/**
     * The server/receiver certificate.
     */
    private OcesX509Certificate serverCertificate;

    /**
     * Constructor.
     * 
     * @param client The client(sender) certificate
     * @param server The server(receiver) certificate
     */
    public Credentials(final OcesX509Certificate client, final OcesX509Certificate server) {
    	clientCertificate = client;
        serverCertificate = server;
    }

    /**
     * Get the server/receiver certificate.
     * @return The server certificate
     */
    public final OcesX509Certificate getServerCertificate() {
        return serverCertificate;
    }
    
    /**
     * Get the client/sender certificate.
     * @return The client certificate
     */
    public final OcesX509Certificate getClientCertificate() {
        return clientCertificate;
    }
}