/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and 
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.communication;

import dk.gov.oiosi.exception.MainException;

public interface IRequest {

	/**
	 * Synchronously sends a request and gets a response
	 * @param message Request message
	 * @return Response message
	 * @throws MainException
	 */
	Response getResponse(OiosiMessage message) throws MainException;

    /**
     * Get the credentials
     * @return The credentials
     */
    Credentials getCredentials();
    
    /**
     * Sets the credentials
     * @param cred The credentials to set for this instance
     */
    void setCredentials(Credentials cred);

    /**
     * Gets the policy corresponding to this Request
     * @return The policy
     */
    SendPolicy getPolicy();
    
    /**
     * Sets the policy corresponding to this Request
     * @param policy The policy to use
     */
    void setPolicy(SendPolicy policy);

    /**
     * Gets the Uri to which the request will be directed
     * @return The Uri
     */
    Uri getRequestUri();
}
