/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.communication;

import java.net.URISyntaxException;
import java.util.Hashtable;
import java.util.Map;

import org.apache.axiom.om.OMElement;

import dk.gov.oiosi.xml.XmlDocument;

/**
 * Response gotten from a RASP http or email endpoint.
 */
public class Response implements IResponse {
    
	/**
     * The actual response message.
     */
    private OiosiMessage responseMessage;
    
    /**
     * Response Uri.
     */
    private Uri responseUri = null;
	
	/**
     * TODO Custom properties.
     */
	private Map<String, Object> properties = new Hashtable<String, Object>();
    
    /**
     * Constructor that takes the actual message body and who it was from as parameter.
     * 
     * @param e Response message body
     * @param from The Uri of the sender
     */
    public Response(OMElement e, String from) {
        // Get the xml body
        XmlDocument messageXml = new XmlDocument(e);

        responseMessage = new OiosiMessage(messageXml);
        
        // Who was the response from?
        if (from != null) {
            try {
				responseUri = new Uri(from);
			} catch (URISyntaxException e1) {
				responseUri = null;
			}
        }
    }

    /**
     * The actual response message.
     * @return received message
     */
    public OiosiMessage getResponseMessage() {
        return responseMessage;
    }

    /**
     * Gets the response URI.
     * @return The response uri
     */
    public Uri getResponseUri() {
        return responseUri;
    }
    
    /**
     * Get the custom properties.
     * 
     * @return The properties
     */
    public Map<String, Object> getProperties() {
        return properties;
    }
}
