/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and 
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.communication;

import dk.gov.oiosi.communication.configuration.mail.IMailServerConfiguration;

/**
 * Representing send policies.
 */
public class SendPolicy {
    
	private RMPolicy mailPolicy = new RMPolicy(2, 60000, false);
    
	private RMPolicy httpPolicy = new RMPolicy(5, 5000, true);
    
    /**
     * Name of WSS policy file.
     */
    private String policyFileName = "./conf/wss_policy.xml";
    
    /**
     * Axis configuration file.
     */
    private String axisConfigurationFile = "./conf/client_axis2.xml";
    
    /**
     * Axis repository.
     */
    private String axisRepsitoryDir = "./repository";
    
    /**
     * The mail inbox configuration. The outbox configuration is defined in the axis2.xml file.
     */
    private IMailServerConfiguration inboxMailConfiguration;
    
    /**
     * Gets the mail reliable messaging policy
     * 
     * @return The mail reliable messaging policy
     */
    public RMPolicy getMailRMPolicy() {
        return mailPolicy;
    }

    /**
     * Sets the mail reliable messaging policy
     * 
     * @param policy The policy
     */
    public void setMailRMPolicy(RMPolicy policy) {
    	mailPolicy = policy;
    }
    
    /**
     * Gets the HTTP reliable messaging policy
     * 
     * @return The HTTP reliable messaging policy
     */
    public RMPolicy getHttpRMPolicy() {
        return httpPolicy;
    }
    
    /**
     * Sets the HTTP reliable messaging policy
     * 
     * @param policy
     */
    public void setHttpRMPolicy(RMPolicy policy) {
        httpPolicy = policy;
    }
    
    /**
     * Sets the name of the WSS policy file.
     * 
     * @param v The policy file location
     */
    public void setWSSPolicyFileName(String v) {
        policyFileName = v;
    }

    /**
     * Gets the name of the WSS policy file.
     * 
     * @return The policy file name
     */
    public String getWSSPolicyFileName() {
        return policyFileName;
    }

    /**
     * Gets the Axis configuration file.
     * 
     * @return axis configuration file
     */
    public String getAxisConfigurationFile() {
        return axisConfigurationFile;
    }

    /**
     * Sets the Axis configuration file.
     * 
     * @param value
     *            axis configuration file
     */
    public void setAxisConfigurationFile(String value) {
        axisConfigurationFile = value;
    }

    /**
     * Gets the Axis repository directory.
     * 
     * @return The Axis repository directory
     */
    public String getAxisRepositoryDir() {
        return axisRepsitoryDir;
    }

    /**
     * Sets the Axis repository directory.
     * 
     * @param value The axis repository
     */
    public void setAxisRepositoryDir(String value) {
        axisRepsitoryDir = value;
    }

    /**
     * Sets the mail configuration for the pop3 server
     * @param inboxMailConfiguration The mail configuration
     */
	public void setInboxMailConfiguration(IMailServerConfiguration inboxMailConfiguration) {
		this.inboxMailConfiguration = inboxMailConfiguration;
	}

	/**
	 * Gets the mail configuration for the pop3 server
	 * @return The mail configuration
	 */
	public IMailServerConfiguration getInboxMailConfiguration() {
		return inboxMailConfiguration;
	}
}