/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.communication;

import java.net.URI;
import java.net.URISyntaxException;

/**
 * Representation of an URI.
 */
public class Uri {

    /**
     * The Uri string.
     */
    private URI value;

    /**
     * Constructor that takes a string representation of the Uri.
     * 
     * @param argument
     *            Uri
     * @throws URISyntaxException 
     */
    public Uri(final String argument) throws URISyntaxException {
        value = new URI(argument);
    }

    /**
     * Gets the Uri as a string.
     * 
     * NOTE. Returns the string value.
     * 
     * @return the Uri as String.
     */
    public String toString() {
        return value.toString();
    }

    /**
     * Gets the absolute Uri as a string.
     * 
     * Note. Returns the string value.
     * 
     * @return the Uri as String.
     */
    public String getAbsoluteUri() {
        return value.toString();
    }

    @Override
    public boolean equals(Object obj) {
    	if (obj == null) return false;
    	
    	if (getClass() != obj.getClass()) return false;
    	Uri other = (Uri) obj;
    	
    	if (!value.equals(other.value)) return false;
    	
    	return true;
    }
    
    @Override
    public int hashCode(){
    	return value.hashCode();
    }
}
