/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and 
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.communication.fault;

/**
 * Enum representing inner SOAP fault code
 */
public enum OiosiInnerFaultCode {
	/**
	 * Fault returned when schema validation fails
	 */
	SchemaValidationFault,
	/**
	 * Fault returned when schematron validation fails
	 */
	SchematronValidationFault,
	/**
	 * Fault returned when the SOAP message signature isn't valid
	 */
	SignatureNotValidFault,
	/**
	 * Fault returned when the document isn't of a known UBL type
	 */
	UnknownDocumentTypeFault,
	/**
	 * Fault returned when message couldn't be persisted
	 */
	MessagePersistencyFault,
	/**
	 * Fault returned when xml transformation fails
	 */
	XsltTransformationFault,
	/**
	 * Fault returned when an unknown internal failure occurs
	 */
	InternalSystemFailureFault,
	/**
	 * Fault returned when an obligatory header is missing
	 */
	MissingHeaderFault,
	DispatchingNotPossibleFault,
	DispatchingDestinationUnknownFault,
	DispatchingFailedFault,
	DispatchingDeniedFault,
	/**
	 * Fault returned when the sender is not authorized in the current context
	 */
	ValidationFault
}
