/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and  
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.communication.service;

import dk.gov.oiosi.communication.OiosiMessage;
import dk.gov.oiosi.exception.MainException;

/**
 * Message handler interface. A generic xml proxy must implement this interface.
 * 
 * The reply with an empty body can be constructed in the following manner:
 * <code>
 * DocumentTypeConfigSearcher documentSearcher = new DocumentTypeConfigSearcher();
 * DocumentTypeConfig documentTypeConfig = documentSearcher.findUniqueDocumentType(xmlDocument);
 * OiosiMessage replyMessage = new OiosiMessage(new XmlDocument());
 * replyMessage.setReplyAction(documentTypeConfig.getEndpointType().getReplyAction());
 * </code>
 */
public interface IServiceContract {

    /**
     * Invoked when service is called.
     * 
     * @param message The incoming message
     * @return Always return reply with empty body (unless testing)
     * @throws MainException
     */
    OiosiMessage requestRespond(OiosiMessage message) throws MainException;

}
