/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 */

package dk.gov.oiosi.configuration;

import java.util.Collection;
import java.util.Iterator;
import java.util.Map;


public class CacheConfigElement
{

	/**
	 * Code of cache client, for which this cache is configured, e.g. SchematronCache or UddiTModelCache
	 */
	private String cacheCode;
	
    /**
     * The namespace and class name
     */
    private String implementationNamespaceClass = "";

    /**
     * The assembly name
     */
    private String implementationAssembly = "";

    /**
     * The list of cache configurations
     */
    private Map<String, String> configurationList = null;

    public CacheConfigElement(String implementationNamespaceClass, String implementationAssembly, Map<String, String> configurationList)
    {
        this("undefined", implementationNamespaceClass, implementationAssembly, configurationList);
    }

    public CacheConfigElement(String cacheCode, String implementationNamespaceClass, String implementationAssembly, Map<String, String> configurationList)
    {
        this.cacheCode = cacheCode;
		this.implementationNamespaceClass = implementationNamespaceClass;
        this.implementationAssembly = implementationAssembly;
        this.configurationList = configurationList;
    }

    /**
     * Gets the implementations classname
     * @return The implementations classname
     */
    public String getImplementationNamespaceClass()
    {
        return this.implementationNamespaceClass;
    }

    /**
     *
     * @param value The implementations classname
     */
    public void setImplementationNamespaceClass(String value)
    {
        this.implementationNamespaceClass = value;
    }

    /**
     * Gets the implementations Assembly
     * @return The implementations classname
     */
    public String getImplementationAssembly()
    {
        return this.implementationAssembly;
    }

    public void setConfigurationList( Map<String, String> value)
    {
        this.configurationList = value;
    }

    public Map<String, String> getConfigurationList()
    {
        return this.configurationList;
    }
    
	public String getCacheCode() {
		return cacheCode;
	}

	public void setCacheCode(String cacheCode) {
		this.cacheCode = cacheCode;
	}    

	@Override
	/**
	 * Generates state of this config only for logging purposes
	 */
	public String toString() {
		final int maxLen = 10;
		StringBuilder b = new StringBuilder();
		b.append("CacheConfig [");
		b.append("code=");
		b.append(cacheCode);
		b.append(", class=");
		b.append(implementationNamespaceClass);
		b.append(", config=");
		b.append(configurationList != null ? toString(configurationList.entrySet(), maxLen) : null);
		b.append("]");
		return b.toString();
	}

	private String toString(Collection<?> collection, int maxLen) {
		StringBuilder builder = new StringBuilder();
		builder.append("[");
		int i = 0;
		for (Iterator<?> iterator = collection.iterator(); iterator.hasNext() && i < maxLen; i++) {
			if (i > 0)
				builder.append(", ");
			builder.append(iterator.next());
		}
		builder.append("]");
		return builder.toString();
	}

	/*
	 * hashCode() and equals() are generated automatically and should be updated
	 * synchronously. Generated automatically by Eclipse.
	 * 
	 * These methods are used at least in unit tests
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((cacheCode == null) ? 0 : cacheCode.hashCode());
		result = prime * result + ((configurationList == null) ? 0 : configurationList.hashCode());
		result = prime * result + ((implementationAssembly == null) ? 0 : implementationAssembly.hashCode());
		result = prime * result + ((implementationNamespaceClass == null) ? 0 : implementationNamespaceClass.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		CacheConfigElement other = (CacheConfigElement) obj;
		if (cacheCode == null) {
			if (other.cacheCode != null)
				return false;
		} else if (!cacheCode.equals(other.cacheCode))
			return false;
		if (configurationList == null) {
			if (other.configurationList != null)
				return false;
		} else if (!configurationList.equals(other.configurationList))
			return false;
		if (implementationAssembly == null) {
			if (other.implementationAssembly != null)
				return false;
		} else if (!implementationAssembly.equals(other.implementationAssembly))
			return false;
		if (implementationNamespaceClass == null) {
			if (other.implementationNamespaceClass != null)
				return false;
		} else if (!implementationNamespaceClass.equals(other.implementationNamespaceClass))
			return false;
		return true;
	}
}
