package dk.gov.oiosi.configuration;

import java.util.HashMap;
import java.util.Map;

import dk.gov.oiosi.common.cache.LeastFrequentlyUsedQuantityCache;
import dk.gov.oiosi.common.cache.MultipleQuantityCache;
import dk.gov.oiosi.common.cache.TimedCache;
import dk.gov.oiosi.security.revocation.crl.CrlLookup;

public class CacheFactoryConstants {
	
	public static final Object[][] DEFAULT_CACHE_CONFIG = new Object[][] {

	{ CacheConfigKey.CertificateCache, TimedCache.class, new String[][] { { TimedCache.CONFIG_VALIDITY_TIME_IN_HOURS, "24" }, { TimedCache.CONFIG_FREQUENCY_IN_HOURS, "1" } } },

	{ CacheConfigKey.OcspLookupCache, TimedCache.class, new String[][]  { { TimedCache.CONFIG_VALIDITY_TIME_IN_HOURS, "1" },  { TimedCache.CONFIG_FREQUENCY_IN_MINUTES, "10" } } },

	{ CacheConfigKey.UddiServiceCache, TimedCache.class, new String[][] { { TimedCache.CONFIG_VALIDITY_TIME_IN_HOURS, "1" }, { TimedCache.CONFIG_FREQUENCY_IN_MINUTES, "10" } } },

	{ CacheConfigKey.UddiTModelCache, TimedCache.class, new String[][]  { { TimedCache.CONFIG_VALIDITY_TIME_IN_HOURS, "24" }, { TimedCache.CONFIG_FREQUENCY_IN_HOURS, "1" } } },

	/*
	 * For now there are 16 OIOUBL schematron files and one OIOXML schematron
	 * file. There is no problem in general to have bigger cache than it is
	 * needed - but the problem happens when cache is less than needed.
	 * 
	 * So let's put 20 as default value.
	 */
	
	{ CacheConfigKey.SchematronCache, LeastFrequentlyUsedQuantityCache.class, new String[][] { { MultipleQuantityCache.CONFIG_MAX_SIZE, "20" } } },

	/*
	 * The same reasons as for Schematron - for now there are only one CRL url, but in future there can be more.
	 */
	
	{ CacheConfigKey.CrlLookupCache, LeastFrequentlyUsedQuantityCache.class, new String[][] { { MultipleQuantityCache.CONFIG_MAX_SIZE, "10" }, { CrlLookup.CONFIG_PARAM_ON_FAILURE_RETRY_INTERVAL_MINUTES, "1" } } },

	{ CacheConfigKey.DEFAULT, LeastFrequentlyUsedQuantityCache.class, new String[][] { { MultipleQuantityCache.CONFIG_MAX_SIZE, "10" } } }, };
	
	public static CacheConfigElement getDefaultCacheConfig(CacheConfigKey cacheKey) {
		String code = null;
		if (cacheKey != null) {
			code = cacheKey.name();
		}
		return getDefaultCacheConfig(code);
	}

	public static CacheConfigElement getDefaultCacheConfig(String cacheCode) {
		CacheConfigElement cacheConfig = null;
		if (cacheCode != null) {
			for (int i = 0; i < DEFAULT_CACHE_CONFIG.length; i++) {
				Object[] config = DEFAULT_CACHE_CONFIG[i];
				CacheConfigKey cacheConfigKey = (CacheConfigKey) config[0];
				if (cacheConfigKey.name().equals(cacheCode)) {
					Class<?> c = (Class<?>) config[1];
					String[][] configList = (String[][]) config[2];
					Map<String, String> list = new HashMap<String, String>();
					for (int j = 0; j < configList.length; j++) {
						list.put(configList[j][0], configList[j][1]);
					}
					cacheConfig = new CacheConfigElement(cacheCode, c.getName(), "dk.gov.oiosi.RaspLibrary", list);
					break;
				}
			}
		}
		return cacheConfig;
	}
	
	public static CacheConfigElement getDefaultCacheConfig() {
		return getDefaultCacheConfig(CacheConfigKey.DEFAULT);
	}
}
