/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.exception;

/**
 * MainException is base for all RaspExceptions.
 */
public class MainException extends Exception {

    /**
     * serialVersionUID.
     */
    private static final long serialVersionUID = 1L;

    /**
     * Constructor.
     * 
     * @param s
     *            message
     * @param e
     *            causing exception
     */
    public MainException(String s, Throwable e) {
        super(s, e);
    }

    /**
     * Constructor.
     * 
     * @param s
     *            message
     */
    public MainException(String s) {
        super(s);
    }

    /**
     * Constructor.
     */
    public MainException() {
        super();
    }

    /**
     * Constructor.
     * 
     * @param innerException
     *            causing exception
     */
    public MainException(Throwable innerException) {
        super(innerException);
    }

    /**
     * Default soap action for service exceptions.
     */
    public static final String DEFAULT_SOAP_ACTION = "InternalSystemFailureFault";

    /**
     * Faults action.
     */
    public String fault = null;

    /**
     * Fault action.
     * 
     * @return fault action
     */
    public String getSoapFaultAction() {
        if (fault == null) {
            return DEFAULT_SOAP_ACTION;
        } else {
            return fault;
        }
    }

    /**
     * Fault action.
     * 
     * @param s
     *            fault action
     */
    public void setSoapFaultAction(String s) {
        fault = s;
    }

    /**
     * Match fault action to thrown exception.
     * 
     * @param s
     *            fault action
     * @return match
     */
    public boolean matchSoapFaultAction(String s) {
        return DEFAULT_SOAP_ACTION.equals(s);
    }

    /**
     * Who to blame.
     */
    private boolean receiverError = true;

    /**
     * Who to blame.
     * 
     * @return who to blame
     */
    public String getSoapFaultCode() {
        if (receiverError) {
            return "Receiver";
        } else {
            return "Sender";
        }
    }

    /**
     * Who to blame.
     * 
     * @param b
     *            blame receiver
     */
    public void setReceiverError(boolean b) {
        receiverError = b;
    }
}
