/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.security.ldap;

import dk.gov.oiosi.security.lookup.CertificateLookupException;

/**
 * The main exception of the Ldap module. Exceptions thrown in this module
 * inherits from this exception, except for those classes that implement
 * interfaces that throws exceptions of a different type.
 * 
 * The exception inherits the MainException from the ExceptionHandeling module.
 * This is to implement that the exception messages are stored in the
 * ErrorMessage.resx file of this module.
 */
public abstract class LdapException extends CertificateLookupException {

	private static final long serialVersionUID = -5831032274164986608L;

	/**
     * Standard default constructor, gives the base constructor the resource
     * manager as parameter.
     */
    public LdapException() {
        super();
    }

    /**
     * Standard constructor that takes a dictionary with keywords as parameter
     * and calls a base constructor with the keywords and the resource manager.
     * 
     * @param e
     *            A dictionary that contains keywords that are used in building
     *            the exception message
     */
    public LdapException(final String e) {
        super(e);
    }

    /**
     * Standard constructor that takes an exception that is the inner exception
     * as parameter and calls the base constructor with both the inner exception
     * and the resource manager.
     * 
     * @param innerException
     *            The inner exception of this exception
     */
    public LdapException(final Throwable innerException) {
        super(innerException);
    }

    /**
     * Constructor that takes a message and the cause exception as parameter and call the base
     * with the exception.
     * 
     * @param message
     *            message
     * @param innerException
     *            cause for trouble
     */
    public LdapException(final String message, final Throwable innerException) {
        super(message, innerException);
    }

}
