/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.security.lookup;

import java.security.cert.X509Certificate;

import dk.gov.oiosi.security.CertificateSubject;
import dk.gov.oiosi.security.validation.CertificateExpiredException;
import dk.gov.oiosi.security.validation.CertificateNotActiveException;

/**
 * Describes an interface where it is possible to get a certificate from the
 * subject string of the certificate.
 */
public interface ICertificateLookup {

    /**
     * Gets a certificate from the subject string and returns a X509Certficate2
     * object of the certificate found.
     * 
     * All exceptions are rooted with the CertificateLookupException
     * 
     * Several things can go wrong in the search and these are represented as
     * the following exceptions CertificateNotFoundException
     * CertificateValidationFailedException MultipleCertificateFoundException
     * SearchFailedException
     * 
     * @param subjectSerialNumber
     *            The subject serial number of the certificate wished
     * @return The certificate with the subject given as parameter
     * @throws CertificateNotFoundException
     * @throws MultipleCertificatesFoundException
     * @throws SearchFailedException
     * @throws CertificateLookupException
     */
    X509Certificate getCertificate(CertificateSubject subjectSerialNumber) throws CertificateNotFoundException,
            MultipleCertificatesFoundException, SearchFailedException, CertificateLookupException, CertificateNotActiveException, CertificateExpiredException;

}
