/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.security.oces;

import java.security.cert.X509Certificate;

import dk.gov.oiosi.security.CertificateHandlingException;

/**
 * Base for OCES Exceptions.
 */
public class OcesCertificateException extends CertificateHandlingException {

    /**
     * serialVersionUID.
     */
    private static final long serialVersionUID = 1L;

    /**
     * Base Constructor.
     */
    public OcesCertificateException() {
    }

    /**
     * Constructor.
     * 
     * @param innerException
     *            causing exception
     */
    public OcesCertificateException(final Exception innerException) {
        super(innerException);
    }

    /**
     * Constructor.
     * 
     * @param e
     *            message
     */
    public OcesCertificateException(final String e) {
        super(e);
    }

    /**
     * Constructor.
     * 
     * @param certificate
     *            certificate
     */
    public OcesCertificateException(final X509Certificate certificate) {
        super(certificate.getIssuerDN() + " : " + certificate.getSubjectDN());
    }

}
