/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.security.oces;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import dk.gov.oiosi.exception.NullOrEmptyArgumentException;

/**
 * Implements the OCES certificate subject key. Ensures that the text that is 
 * used to detect certificate strings.
 */
public class OcesCertificateSubjectKey {

	private static Pattern subjectKeyStringPattern = Pattern.compile("(\\s)+");
	
	private String _subjectKeyString;

    /**
     * Default constructor
     */
    public OcesCertificateSubjectKey() {
        _subjectKeyString = "NOID";
    }

    /**
     * Constructor that takes the subject key string as parameter.
     * @param subjectKeyString
     * @throws InvalidOcesCertificateSubjectKeyException
     */
    public OcesCertificateSubjectKey(String subjectKeyString) throws InvalidOcesCertificateSubjectKeyException {
        CheckSubjectKeyString(subjectKeyString);
        _subjectKeyString = subjectKeyString;
    }

    /**
     * Gets the subject key string.
     * @return The subject key string
     */
    public String getSubjectKeyString() { 
        return _subjectKeyString;
    }
    
    /**
     * Sets the subject key string for this certificate
     * @param value The subject key string
     * @throws InvalidOcesCertificateSubjectKeyException
     */
    public void setSubjectKeyString(String value) throws InvalidOcesCertificateSubjectKeyException {
	    CheckSubjectKeyString(value);
	    _subjectKeyString = value;
    }
    
    private void CheckSubjectKeyString(String subjectKeyString) throws InvalidOcesCertificateSubjectKeyException {
        if (subjectKeyString == null || subjectKeyString.equals("")) throw new NullOrEmptyArgumentException("keyString");
        
        Matcher m = subjectKeyStringPattern.matcher(subjectKeyString);

        if (m.matches())
        {
            throw new InvalidOcesCertificateSubjectKeyException("Invalid subject key string.");
        }
    }
}
