/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.security.revocation;

import java.net.URL;

/**
 * Custom exception used when an unexpected exception is thrown during a check
 * to see if the certificate has been revoked.
 */
public class CheckCertificateRevokedUnexpectedException extends RevocationException {

    private static final long serialVersionUID = 5388451726324764807L;

    /**
     * URL to CRL for which exception occurred.
     *
     * @since OIORASP 1.3.0
     */
    private URL crlUrl;

    /**
     * Constructor with inner exception.
     *
     * @param message error message
     * @since OIORASP 1.3.0
     */
    public CheckCertificateRevokedUnexpectedException(final String message) {
        super(message);
    }

    /**
     * Constructor with inner exception.
     *
     * @param innerException inner exception of the thrown exception
     * @deprecated from 1.3.0: Use CheckCertificateRevokedUnexpectedException(final String message, final Exception innerException)
     */
    public CheckCertificateRevokedUnexpectedException(final Exception innerException) {
        super(innerException);
    }

    /**
     * Constructor with inner exception.
     *
     * @param message        error message
     * @param innerException inner exception of the thrown exception
     * @since OIORASP 1.3.0
     */
    public CheckCertificateRevokedUnexpectedException(final String message, final Exception innerException) {
        super(message, innerException);
    }

    /**
     * Constructor with inner exception.
     *
     * @param message        error message
     * @param innerException inner exception of the thrown exception
     * @param crlUrl         URL to CRL server, which initiated problem
     * @since OIORASP 1.3.0
     */
    public CheckCertificateRevokedUnexpectedException(final String message, final Exception innerException, URL crlUrl) {
        super(message, innerException);
        this.crlUrl = crlUrl;
    }

    /**
     * @return URL to CRL for which exception occurred
     * @since OIORASP 1.3.0
     */
    public URL getCrlUrl() {
        return crlUrl;
    }
}