/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.security.revocation;

import java.security.cert.X509Certificate;

/**
 * A class implementing this interface is capable of checking whether a
 * X509 certificate has been revoked.
 */
public interface IRevocationLookup {

    public static enum RevocationSourceType {
        CRL, OCSP
    }

    /**
     * Checks whether the certificate has been revoked.
     *
     * @param certificate The certificate to check
     * @return Returns the check status
     * @throws RevocationException On error...
     */
    RevocationResponse checkCertificate(X509Certificate certificate) throws RevocationException;

    /**
     * Provides information about type of source, which is used by
     * implementation to get information about certificate revocation
     *
     * @return revocation source type
     * @since OIORASP 1.3.0
     */
    /*
      * dlk 2012-04-19
      *
      * Potentially it can lead to problems with backward compatibility, if some
      * customers implemented their own revocation lookup clients.
      *
      * But strategically, it is more correct to use such method in case if in
      * future there will be more than these 3 kinds of implementations -
      * RevocationSourceType will be just extended with DB or OCSP2 or whatever.
      */
    RevocationSourceType getRevocationSourceType();
}