/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.security.revocation;

import java.util.Date;

/**
 * This class is used to store the result of a revocation certification validation
 * process.
 */
public class RevocationResponse {

    /**
     * This property is used to indicate if the certificate has been revoked
     * (either permanently or temporarily (on hold)).
     */
    private boolean isValid;

    /**
     * This property is used to store the time at or before which newer
     * information will be available about the status of the certificate.
     */
    private Date nextUpdate = new Date(0);


    public RevocationResponse() {
    }

    public RevocationResponse(boolean valid, Date nextUpdate) {
        this.isValid = valid;
        this.nextUpdate = nextUpdate;
    }

    /**
     * Gets the next update of the revocation check of the certificate.
     *
     * @return The next update date
     */
    public Date getNextUpdate() {
        return nextUpdate;
    }

    /**
     * Sets the next update of the revocation check of the certificate.
     *
     * @param value The next update date
     */
    public void setNextUpdate(Date value) {
        nextUpdate = value;
    }

    /**
     * Gets whether the certificate is valid.
     *
     * @return True, if valid
     */
    public boolean isValid() {
        return isValid;
    }

    /**
     * Sets whether the certificate is valid.
     *
     * @param value True if valid, false if not
     */
    public void setIsValid(boolean value) {
        isValid = value;
    }

    @Override
    public String toString() {
        return "Is valid: " + isValid + ", Next update: " + nextUpdate;
    }
}