/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.security.validation;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.security.cert.X509Certificate;
import java.util.Date;

/**
 * TODO: Verify this class. Class to validate X509 certificates.
 */
public class CertificateValidator {

    private static final Log log = LogFactory.getLog(CertificateValidator.class);

    /**
     * Constructor.
     */
    private CertificateValidator() {
    }

    /**
     * Attempts to validate the certificate. If the certificate is invalid an exception is thrown.
     * <p/>
     * It only validates what can be done locally without the CTL list to avoid dependency on downloading the CTL to validate
     * a certificate.
     * <p/>
     * To check whether the certificate is trusted use the Ocsp module instead.
     *
     * @param certificate certificate
     * @throws CertificateNotActiveException On error...
     * @throws CertificateExpiredException   On error...
     */
    public static void validateCertificate(X509Certificate certificate) throws CertificateNotActiveException, CertificateExpiredException {
        /*
         * TODO: Better checks than just expiration date
         * X509Chain chain = CreateChain(certificate); //Modified chain
         * validation of the certificate. We are not interested in Ctl lists
         * foreach (X509ChainStatus status in chain.ChainStatus) { switch
         * (status.Status) { case X509ChainStatusFlags.CtlNotSignatureValid:
         * case X509ChainStatusFlags.CtlNotTimeValid: case
         * X509ChainStatusFlags.CtlNotValidForUsage: case
         * X509ChainStatusFlags.NoError: case
         * X509ChainStatusFlags.RevocationStatusUnknown: case
         * X509ChainStatusFlags.OfflineRevocation: break; default: throw new
         * CertificateFailedChainValidationException(status); } }
         */
        checkCertificateActivated(certificate);
        checkCertificateExpired(certificate);
    }

    /**
     * Checks if the certificate is active now.
     *
     * @param certificate The certificate to check
     * @throws CertificateNotActiveException If activation time is after now.
     */
    public static void checkCertificateActivated(X509Certificate certificate) throws CertificateNotActiveException {
        if (certificate.getNotBefore().after(new Date())) {
            if (log.isDebugEnabled()) {
                log.debug("Certificate [" + certificate.getSubjectDN().getName() + "] is not active now [" + new Date() + "]");
            }
            throw new CertificateNotActiveException(certificate.getNotBefore(), certificate.getSubjectDN().getName());
        }
    }

    /**
     * Checks if the certificate is expired
     *
     * @param certificate The certificate to check
     * @throws CertificateExpiredException On error...
     */
    public static void checkCertificateExpired(X509Certificate certificate) throws CertificateExpiredException {
        if (certificate.getNotAfter().before(new Date())) {
            throw new CertificateExpiredException(certificate.getNotAfter(), certificate.getSubjectDN().getName());
        }
    }
}