/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.uddi;

/**
 * Represents the endpoint key type code for the following four codes;
 * EAN, OVT, CVR, P, SE, VANS, IBAN, DUNS, CPR and other.
 */
public class EndpointKeyTypeCode {
    

	private int value = EndpointKeyTypeCode.other;
    /**
     * Constant value that defines EAN number
     */
	
    public static final int ean = 0;
    /**
     * Constant value that defines OVT number
     */
    public static final int ovt = 1;
    /**
     * Constant value that defines CVR number
     */
    public static final int cvr = 2;
    /**
     * Constant value that defines P number
     */
    public static final int p = 3;
    /**
     * Constant value that defines SE number
     */
    public static final int se = 4;
    /**
     * Constant value that defines VANS number
     */
    public static final int vans = 5;
    /**
     * Constant value that defines IBAN number
     */
    public static final int iban = 6;
    /**
     * Constant value that defines DUNS number
     */
    public static final int duns = 7;
    /**
     * Constant value that defines CPR number
     */
    public static final int cpr = 8;
    /**
     * Constant value that defines other
     */
    public static final int other = -1;
    
    /**
     * Constructor that takes the key type code.
     * 
     * @param v
     */
    public EndpointKeyTypeCode(int v) {
        value = v;
    }

    /**
     * Constructor that takes the key type as a string.
     * 
     * @param type
     */
    public EndpointKeyTypeCode(String type) {
               if ("ean".equalsIgnoreCase(type)) {
            value = ean;
        } else if ("cvr".equalsIgnoreCase(type)) {
            value = cvr;
        } else if ("ovt".equalsIgnoreCase(type)) {
            value = ovt;
        } else if ("p".equalsIgnoreCase(type)) {
            value = p;
        } else if ("se".equalsIgnoreCase(type)) {
            value = se;
        } else if ("vans".equalsIgnoreCase(type)) {
            value = vans;
        } else if ("iban".equalsIgnoreCase(type)) {
            value = iban;
        } else if ("duns".equalsIgnoreCase(type)) {
            value = duns;
        }  else if ("cpr".equalsIgnoreCase(type)) {
            value = cpr;
        } else {
            value = other;
        }
    }

    /**
     * Get the key type code.
     * 
     * @return The key type code
     */
    public int getCode() {
        return value;
    }

    /**
     * Gets the endpoint key type code for EAN number.
     * 
     * @return The EndpointKeyTypeCode corresponding to EAN
     */
    public static EndpointKeyTypeCode getEan() {
        return new EndpointKeyTypeCode(ean);
    }

    /**
     * Gets the endpoint key type code for OVT number.
     * 
     * @return The EndpointKeyTypeCode corresponding to OVT
     */
    public static EndpointKeyTypeCode getOvt() {
        return new EndpointKeyTypeCode(ovt);
    }

    /**
     * Gets the endpoint key type code for CVR number.
     * 
     * @return The EndpointKeyTypeCode corresponding to CVR
     */
    public static EndpointKeyTypeCode getCvr() {
        return new EndpointKeyTypeCode(cvr);
    }
    
    /**
     * Gets the endpoint key type code for P number.
     * 
     * @return The EndpointKeyTypeCode corresponding to P
     */
    public static EndpointKeyTypeCode getP() {
        return new EndpointKeyTypeCode(p);
    }
    
    /**
     * Gets the endpoint key type code for SE number.
     * 
     * @return The EndpointKeyTypeCode corresponding to SE
     */
    public static EndpointKeyTypeCode getSe() {
        return new EndpointKeyTypeCode(se);
    }
    
    /**
     * Gets the endpoint key type code for VANS number.
     * 
     * @return The EndpointKeyTypeCode corresponding to VANS
     */
    public static EndpointKeyTypeCode getVans() {
        return new EndpointKeyTypeCode(vans);
    }
    
    /**
     * Gets the endpoint key type code for IBAN number.
     * 
     * @return The EndpointKeyTypeCode corresponding to IBAN
     */
    public static EndpointKeyTypeCode getIban() {
        return new EndpointKeyTypeCode(iban);
    }
    
    /**
     * Gets the endpoint key type code for DUNS number.
     * 
     * @return The EndpointKeyTypeCode corresponding to DUNS
     */
    public static EndpointKeyTypeCode getDuns() {
        return new EndpointKeyTypeCode(duns);
    }
    
    /**
     * Gets the endpoint key type code for CPR number.
     * 
     * @return The EndpointKeyTypeCode corresponding to CPR
     */
    public static EndpointKeyTypeCode getCpr() {
        return new EndpointKeyTypeCode(cpr);
    }

    /**
     * Gets the endpoint key type code for other.
     * 
     * @return The EndpointKeyTypeCode corresponding to "Other"
     */
    public static EndpointKeyTypeCode getOther() {
        return new EndpointKeyTypeCode(other);
    }
}
