package dk.gov.oiosi.uddi;

/**
 * Class that contains information about a process definition
 */
public class ProcessDefinition {

    /**
     * The identifier of the process definition in the UDDI register
     */
    private UddiId Id;

    /**
     * The name of the process definition
     */
    private String Name;

    /**
     * The description of the process definition
     */
    private String Description;

    /**
     * The business process identifier
     */
    private String Identifier;

    /**
     * The business process identifier type
     */
    private String IdentifierType;

    /**
     * The registration conformance claim
     */
    private String RegistrationConformanceClaim;

    /**
     * Constructor that takes the identifier, the name and the description of the process definition.
     *
     * @param id
     * @param name
     * @param description
     * @param identifier
     * @param identifierType
     * @param registrationConformanceClaim
     */
    public ProcessDefinition(UddiId id, String name, String description, String identifier, String identifierType, String registrationConformanceClaim) {
        Id = id;
        Name = name;
        Description = description;
        Identifier = identifier;
        IdentifierType = identifierType;
        RegistrationConformanceClaim = registrationConformanceClaim;
    }

    public UddiId getId() {
        return Id;
    }

    public String getName() {
        return Name;
    }

    public String getDescription() {
        return Description;
    }

    public String getIdentifier() {
        return Identifier;
    }

    public String getIdentifierType() {
        return IdentifierType;
    }

    public String getRegistrationConformanceClaim() {
        return RegistrationConformanceClaim;
    }

    @Override
    public String toString() {
        return "ProcessDefinition{" +
                "Id=" + Id +
                ", Name='" + Name + '\'' +
                ", Description='" + Description + '\'' +
                ", Identifier='" + Identifier + '\'' +
                ", IdentifierType='" + IdentifierType + '\'' +
                ", RegistrationConformanceClaim='" + RegistrationConformanceClaim + '\'' +
                '}';
    }
}