/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.uddi;

import dk.gov.oiosi.common.OutVariable;
import dk.gov.oiosi.exception.NullOrEmptyArgumentException;
import dk.itst.uddi.client.types.core.CategoryBag;
import dk.itst.uddi.client.types.core.KeyedReference;
import dk.itst.uddi.client.types.core.KeyedReferenceGroup;

import java.util.Hashtable;
import java.util.Map;

public class UddiCategoryBag {

    private CategoryBag bag;
    private Map<String, KeyedReference> keyedReferenceBag;
    private Map<String, KeyedReferenceGroup> keyedReferenceGroupBag;


    public UddiCategoryBag(CategoryBag categoryBag) {

        bag = categoryBag;

        keyedReferenceBag = new Hashtable<String, KeyedReference>();
        keyedReferenceGroupBag = new Hashtable<String, KeyedReferenceGroup>();

        if (categoryBag.getKeyedReferenceArray() != null) {
            for (KeyedReference keyedReference : categoryBag.getKeyedReferenceArray()) {
                keyedReferenceBag.put(keyedReference.getTModelKey().toLowerCase(), keyedReference);
            }
        }

        if (categoryBag.getKeyedReferenceGroupArray() != null) {
            for (KeyedReferenceGroup keyedReferenceGroup : categoryBag.getKeyedReferenceGroupArray()) {
                keyedReferenceGroupBag.put(keyedReferenceGroup.getTModelKey().toLowerCase(), keyedReferenceGroup);
            }
        }
    }

    public CategoryBag getCategoryBag() {
        return bag;
    }

    public boolean tryGetKeyedReference(String tModelKey, OutVariable<KeyedReference> keyedRef) {
        if (tModelKey == null || tModelKey.length() <= 0) {
            throw new NullOrEmptyArgumentException("tModelKey");
        }
        keyedRef.setVariable(keyedReferenceBag.get(tModelKey.toLowerCase()));

        return keyedRef.getVariable() != null;
    }

    public boolean tryGetKeyedReferenceGroup(String tModelKey, OutVariable<KeyedReferenceGroup> keyedRef) {
        if (tModelKey == null || tModelKey.length() <= 0) {
            throw new NullOrEmptyArgumentException("tModelKey");
        }
        keyedRef.setVariable(keyedReferenceGroupBag.get(tModelKey.toLowerCase()));

        return keyedRef.getVariable() != null;
    }

    /*public static KeyedReference GetMandatoryCategoryByIdentifier(CategoryBag bag, String categoryIdentifier) throws Exception {
        return GetCategoryByIdentifier(bag, categoryIdentifier);
    }

    public static KeyedReference GetOptionalCategoryByIdentifier(CategoryBag bag, String categoryIdentifier) {
        try {
            KeyedReference category = GetCategoryByIdentifier(bag, categoryIdentifier);
            return category;
        }
        catch(Exception ex) {
            return null;
        }
    }

    public static KeyedReference GetOptionalCategoryByIdentifier(IdentifierBag bag, String categoryIdentifier) {
        try {
            KeyedReference category = GetCategoryByIdentifier(bag, categoryIdentifier);
            return category;
        }
        catch(Exception ex) {
            return null;
        }
    }

    private static KeyedReference GetCategoryByIdentifier(CategoryBag bag, String categoryIdentifier) throws Exception {
        if (bag == null) throw new ArgumentNullException("bag");
        if (bag.getKeyedReferenceArray() == null) throw new Exception("No category with " + categoryIdentifier + " found.");

        return GetCategoryByIdentifier(bag.getKeyedReferenceArray(), categoryIdentifier);
    }

    private static KeyedReference GetCategoryByIdentifier(IdentifierBag bag, String categoryIdentifier) throws Exception {
        if (bag == null) throw new ArgumentNullException("bag");
        if (bag.getKeyedReferenceArray() == null) throw new Exception("No category with " + categoryIdentifier + " found.");

        return GetCategoryByIdentifier(bag.getKeyedReferenceArray(), categoryIdentifier);
    }


    private static KeyedReference GetCategoryByIdentifier(Object[] keyedReferences, String categoryIdentifier) throws Exception {
        List<KeyedReference> keyedReferenceList = new LinkedList<KeyedReference>();
        for (Object category : keyedReferences) {
            //if the category is a keyed reference group ignore it.
            if (!(category instanceof KeyedReference)) continue;
            KeyedReference keyRef = (KeyedReference)category;
            if (keyRef.getTModelKey().toLowerCase().equals(categoryIdentifier.toLowerCase())) {
                keyedReferenceList.add(keyRef);
            }
        }

        if (keyedReferenceList == null || keyedReferenceList.size() == 0) throw new Exception("No category with " + categoryIdentifier + " found.");
        if (keyedReferenceList.size() > 1) throw new Exception("More than one category found: " + categoryIdentifier);

        return keyedReferenceList.get(0);
    }*/

    @Override
    public String toString() {
        return "UddiCategoryBag{" +
                "bag=" + bag +
                ", keyedReferenceBag=" + keyedReferenceBag +
                ", keyedReferenceGroupBag=" + keyedReferenceGroupBag +
                '}';
    }
}