/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.util;

import java.security.cert.X509Certificate;
import java.util.Date;
import java.util.List;
import java.util.Vector;

import org.apache.ws.security.WSConstants;
import org.apache.ws.security.WSSecurityEngineResult;
import org.apache.ws.security.handler.WSHandlerResult;

/**
 * Util to extract certificate from Axis Rampart internal structures.
 */
public class AxisWSSUtil {

    /**
     * Extract certificate.
     *
     * @param o object
     * @return certificate
     */
    @SuppressWarnings({ "unchecked", "deprecation" })
	public static X509Certificate getCertificateFrom_RECV_RESULTS(Object o) {
        if (o instanceof List) {
            List<WSHandlerResult> l = (List<WSHandlerResult>) o;
            for (int i = 0; i < l.size(); i++) {
                WSHandlerResult handlerResult = l.get(i);
                Vector<WSSecurityEngineResult> v = handlerResult.getResults();
                for (int j = 0; j < v.size(); j++) {
                    WSSecurityEngineResult securityResult = v.get(j);
                    switch (securityResult.getAction()) {
                        case WSConstants.SIGN:
                            return securityResult.getCertificate();
                        case WSConstants.ENCR:
                            break;
                        case WSConstants.TS:
                            break;
                        default:
                            break;
                    }
                }
            }
        }
        return null;
    }

    /**
     * Check if message was encrypted.
     *
     * @param o object
     * @return certificate
     */
    @SuppressWarnings({ "rawtypes", "deprecation" })
	public static boolean getMessageEncryptedFrom_RECV_RESULTS(Object o) {
        if (o instanceof List) {
            List l = (List) o;
            for (int i = 0; i < l.size(); i++) {
                WSHandlerResult handlerResult = (WSHandlerResult) l.get(i);
                Vector v = handlerResult.getResults();
                for (int j = 0; j < v.size(); j++) {
                    // System.out.println(" - - " + v.get( j
                    // ).getClass().getCanonicalName() );
                    WSSecurityEngineResult securityResult = (WSSecurityEngineResult) v.get(j);
                    switch (securityResult.getAction()) {
                        case WSConstants.SIGN:
                            // System.out.println(" - - : action == SIGN" );
                            break;

                        case WSConstants.ENCR:
                            //System.out.println(" - - : action == ENCR" );
                            return true;

                        case WSConstants.TS:
                            // System.out.println(" - - : action == TS " );
                            break;
                        default:
                            // System.out.println(" - - : action == " +
                            // securityResult.getAction() );
                            break;
                    }
                }
            }
        }
        return false;
    }

    /**
     * Get Timestamp Created from soap message.
     *
     * @param o object
     * @return certificate
     */
    @SuppressWarnings({ "deprecation", "rawtypes" })
	public static Date getTimestampCreatedFrom_RECV_RESULTS(Object o) {
        if (o instanceof List) {
            List l = (List) o;
            for (int i = 0; i < l.size(); i++) {
                WSHandlerResult handlerResult = (WSHandlerResult) l.get(i);
                Vector v = handlerResult.getResults();
                for (int j = 0; j < v.size(); j++) {
                    // System.out.println(" - - " + v.get( j
                    // ).getClass().getCanonicalName() );
                    WSSecurityEngineResult securityResult = (WSSecurityEngineResult) v.get(j);
                    switch (securityResult.getAction()) {
                        case WSConstants.SIGN:
                            // System.out.println(" - - : action == SIGN" );
                            break;

                        case WSConstants.ENCR:
                            //System.out.println(" - - : action == ENCR" );
                            break;

                        case WSConstants.TS:
                            Date created = securityResult.getTimestamp().getCreated().getTime();
                            return created;
                        default:
                            // System.out.println(" - - : action == " +
                            // securityResult.getAction() );
                            break;
                    }
                }
            }
        }
        return null;
    }
    
}