/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.xml;

import dk.gov.oiosi.exception.NullOrEmptyArgumentException;

/**
 * Represents an XML Schema.
 * <p/>
 * Wraps the Java STD. w3 and Axis Axiom document representation.
 */
public class XmlSchema extends XmlDocument {

    /**
     * Constructs a new XmlSchema class from a given file.
     *
     * @param file File name of schema file
     */
    public XmlSchema(String file) {
        if (file == null || file.trim().isEmpty()) {
            throw new NullOrEmptyArgumentException("Xml schema file name");
        }
        fileName = file;
    }

    /**
     * Return name of schema file.
     *
     * @return name of schema file
     */
    @Override
    public String getFileName() {
        return fileName;
    }
}