/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and 
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.xml.schematron;

import dk.gov.oiosi.xml.XmlException;

/**
 * Exception thrown if the load of the schematron stylesheet fails. 
 */
public class FailedToLoadSchematronStylesheetException extends XmlException {
	private static final long serialVersionUID = 2L;
	
	private String schematronPath = ""; 
	
	/**
	 * Constructor that takes the schematron stylesheet path used when attempting 
	 * to load the stylesheet and the exception caught as inner exception.
	 * @param schematronPath
	 * @param innerException
	 */
	public FailedToLoadSchematronStylesheetException(String schematronPath, Exception innerException) {
		super("Failed to load the schematron stylesheet, tried the following path: " + schematronPath, innerException);
		this.schematronPath = schematronPath; 
	}
	
	/**
	 * Gets the schematron path used when the load failed.
	 * @return The schematron path
	 */
	public String getSchematronPath() {
		return schematronPath;
	}
}
