/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and 
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.xml.schematron;

import javax.xml.transform.Transformer;

import dk.gov.oiosi.common.OutVariable;
import dk.gov.oiosi.common.cache.IThreadSafeCache;
import dk.gov.oiosi.configuration.CacheFactory;
import dk.gov.oiosi.exception.ArgumentNullException;
import dk.gov.oiosi.xml.xslt.XsltUtility;

public class SchematronStore
{

    private IThreadSafeCache<String, Transformer> _cache;
    
    private static final XsltUtility UTILITY = new XsltUtility();

    /**
     * Constructor that uses a default of max two coherent compiled stylesheeets
     * in memory.
     */
    public SchematronStore() {
        _cache = CacheFactory.getInstance().getSchematronStoreCache();
    }

    /**
     * Constructor that takes the configuration for the schematron store
     * 
     * @deprecated Use SchematronStoreFactory instead.
     */
    @SuppressWarnings("unused")
	public SchematronStore(ISchematronStoreConfig configuration) {
    	super();
    } 

    /**
     * Gets the compiled schematron stylesheet from a given path.
     * @param schematronPath
     */
    public Transformer getCompiledSchematron(String schematronPath) throws FailedToLoadSchematronStylesheetException {
        if (schematronPath == null)
        {
            throw new ArgumentNullException("path");
        }

        Transformer compiledSchematron = null;

    	OutVariable<Transformer> result = new OutVariable<Transformer>(); 
        if (_cache.tryGetValue(schematronPath, result))
        {
            compiledSchematron = result.getVariable();
        }
        else
        {
            compiledSchematron = loadCompiledSchematron(schematronPath);
            _cache.add(schematronPath, compiledSchematron);
        }

        return compiledSchematron;
    }

    private Transformer loadCompiledSchematron(String path) throws FailedToLoadSchematronStylesheetException {
        try {
            return UTILITY.precompiledStyleSheet(path);
        } catch (Exception ex) {
            throw new FailedToLoadSchematronStylesheetException(path, ex);
        }
    }
}
