/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.xml.schematron;

import dk.gov.oiosi.xml.XmlDocument;

/**
 * Custom exception used when schematron validation fails.
 */
public class SchematronValidationFailedException extends SchematronValidationException {

    /**
     * serialVersionUID.
     */
    private static final long serialVersionUID = 2L;

    /**
     * Soap fault sub type / RASP fault action.
     */
    public static final String FAULT_ACTION = "SchematronValidationFault";

    /**
     * Constructor.
     * 
     * @param document
     *            the current document
     * @param innerException
     *            innerexception of the thrown exception
     */
    public SchematronValidationFailedException(final XmlDocument document, final Exception innerException) {
        super("Failed to Schematron validate document : " + innerException.getMessage());
        setReceiverError(false);
        setSoapFaultAction(FAULT_ACTION);
    }

    /**
     * Constructor.
     * 
     * @param reason
     */
    public SchematronValidationFailedException(final String reason) {
        super(reason);
        setReceiverError(false);
        setSoapFaultAction(FAULT_ACTION);
    }

    /**
     * Constructor with cause exception and error message
     * 
     * @param reason
     */
    public SchematronValidationFailedException(final String reason, final Exception cause) {
        super(reason, cause);
        setReceiverError(false);
        setSoapFaultAction(FAULT_ACTION);
    }
}
