package dk.gov.oiosi.xml.xpath;

import org.apache.axiom.om.OMAttribute;
import org.apache.axiom.om.OMDocument;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.xpath.DocumentNavigator;
import org.apache.commons.collections.iterators.ArrayIterator;
import org.jaxen.JaxenConstants;
import org.jaxen.NamedAccessNavigator;
import org.jaxen.UnsupportedAxisException;

import javax.xml.namespace.QName;
import java.util.Iterator;

/**
 * This class extends standard DocumentNavigator with NamedAccessNavigator
 * implementations, which is more efficient for AXIOM model,
 *
 * as it can call getFirstChildWithName at give node instead of searching for
 * all potential matches. This navigator should be used in case if AXIOM
 * document is not built to avoid its building during XPath calculation.
 *
 * @author Dmitriy Lapko, TrueLink A/S
 * @since OIORASP 1.3.0
 */
public class NamedOMDocumentNavigator extends DocumentNavigator implements NamedAccessNavigator {

	private static final long serialVersionUID = 1690535293994714808L;

	@Override
	public Iterator<?> getChildAxisIterator(Object contextNode, String localName, String namespacePrefix, String namespaceURI) throws UnsupportedAxisException {
		if (contextNode == null) {
			return JaxenConstants.EMPTY_ITERATOR;
		}
		OMElement contextElement;
		if (contextNode instanceof OMDocument) {
			contextElement = ((OMDocument) contextNode).getOMDocumentElement();
		} else if (contextNode instanceof OMElement) {
			contextElement = (OMElement) contextNode;
		} else {
			throw new UnsupportedAxisException("Unsupported contextNode type: " + contextNode.getClass());
		}
		QName qname = new QName(namespaceURI, localName, namespacePrefix);
		OMElement firstChildWithName = contextElement.getFirstChildWithName(qname);
		return new ArrayIterator(new OMElement[] { firstChildWithName });
	}

	@Override
	public Iterator<?> getAttributeAxisIterator(Object contextNode, String localName, String namespacePrefix, String namespaceURI) throws UnsupportedAxisException {
		if (contextNode == null) {
			return JaxenConstants.EMPTY_ITERATOR;
		}
		OMElement contextElement;
		if (contextNode instanceof OMDocument) {
			contextElement = ((OMDocument) contextNode).getOMDocumentElement();
		} else if (contextNode instanceof OMElement) {
			contextElement = (OMElement) contextNode;
		} else {
			throw new UnsupportedAxisException("Unsupported contextNode type: " + contextNode.getClass());
		}
		QName qname = new QName(namespaceURI, localName, namespacePrefix);
		OMAttribute attribute = contextElement.getAttribute(qname);
		return new ArrayIterator(new OMAttribute[] { attribute });
	}

}
