/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.xml.xpath;

/**
 * Representation of XML namespace for XPath search.
 */
public final class PrefixedNamespace {

    /**
     * The namespace.
     */
    private String namespace;

    /**
     * The prefix.
     */
    private String prefix;

    /**
     * Default constructor.
     */
    public PrefixedNamespace() { }

    /**
     * Constructor that takes the namespace and prefix.
     *
     * @param ns
     *            Namespace
     * @param pre
     *            Prefix
     */
    public PrefixedNamespace(final String ns, final String pre) {
        namespace = ns;
        prefix = pre;
    }

    /**
     * Gets the namespace.
     *
     * @return namespace
     */
    public String getNamespace() {
        return namespace;
    }

    /**
     * Sets the namespace.
     *
     * @param value
     *            namespace
     */
    public void setNamespace(final String value) {
        namespace = value;
    }

    /**
     * Gets the namespace prefix.
     *
     * @return prefix
     */
    public String getPrefix() {
        return prefix;
    }

    /**
     * Sets the namespace prefix.
     *
     * @param value
     *            prefix
     */
    public void setPrefix(final String value) {
        prefix = value;
    }
}
