/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.xml.xpath.discriminator;

/**
 * Configuration to an XPath discrimination.
 */
public final class XPathDiscriminatorConfig {

    /**
     * the XPath search expression.
     */
    private String xpathExpression = "";

    /**
     * the result to match.
     */
    private String xpathExpectedResult = "";

    /**
     * Constructor.
     */
    public XPathDiscriminatorConfig() {
    }

    /**
     * Constructor that takes the XPath expression and the expected result as
     * paramters.
     *
     * @param expression
     *            XPath search expression
     * @param expectedResult
     *            Expected result
     */
    public XPathDiscriminatorConfig(final String expression, final String expectedResult) {
        xpathExpression = expression;
        xpathExpectedResult = expectedResult;
    }

    /**
     * Gets and sets the XPath expression used by the XPath discriminator.
     *
     * @return XPath search expression
     */
    public String getXPathExpression() {
        return xpathExpression;
    }

    /**
     * Gets and sets the XPath expression used by the XPath discriminator.
     *
     * @param value
     *            XPath search expression
     */
    public void setXPathExpression(final String value) {
        xpathExpression = value;
    }

    /**
     * Gets and sets the XPath expected result used by the XPath discriminator.
     *
     * @return Expected result
     */
    public String getXPathExpectedResult() {
        return xpathExpectedResult;
    }

    /**
     * Gets and sets the XPath expected result used by the XPath discriminator.
     *
     * @param value
     *            Expected result
     */
    public void setXPathExpectedResult(final String value) {
        xpathExpectedResult = value;
    }

    /**
     * Returns whether the other XPathDiscriminator given is equals to this.
     *
     * @param other
     *            the object to be checked
     * @return true if equals
     */
    public boolean equals(final Object other) {
        if (other == null) {
            return false;
        }
        if (!(other instanceof XPathDiscriminatorConfig)) {
            return false;
        }
        if (!xpathExpression.equals(((XPathDiscriminatorConfig) other).xpathExpression)) {
            return false;
        }
        return xpathExpectedResult.equals(((XPathDiscriminatorConfig) other).xpathExpectedResult);
    }

}
