package dk.gov.oiosi.common;

import junit.framework.TestCase;

import java.util.concurrent.TimeUnit;

public class TimeSpanTest extends TestCase {

	public void testTimeSpan() {
		TimeSpan t = new TimeSpan();
		assertEquals(0, t.getSpanMillis());
		assertEquals(0, t.getSpanMinutes());
		assertEquals("0 MILLISECONDS", t.toString());
		t.setHours(10);
		assertEquals("10 HOURS", t.toString());
		assertEquals(10 * 60 * 60 * 1000, t.getSpanMillis());
		assertEquals(10 * 60, t.getSpanMinutes());
	}

	public void testTimeSpanTimeUnitLong() {
		TimeSpan t = new TimeSpan(TimeUnit.DAYS, 1);
		assertEquals(24 * 60 * 60 * 1000, t.getSpanMillis());
		assertEquals(24 * 60, t.getSpanMinutes());
		assertEquals(TimeUnit.DAYS, t.getTimeUnit());
		assertEquals(1, t.getTimeUnitSpan());
		assertEquals("1 DAYS", t.toString());

		t = new TimeSpan(TimeUnit.HOURS, 24);
		assertEquals(24 * 60 * 60 * 1000, t.getSpanMillis());
		assertEquals(24 * 60, t.getSpanMinutes());
		assertEquals(TimeUnit.HOURS, t.getTimeUnit());
		assertEquals(24, t.getTimeUnitSpan());
		assertEquals("24 HOURS", t.toString());
	}

	public void testTimeSpanLong() {
		TimeSpan t = new TimeSpan(1);
		assertEquals(1, t.getSpanMillis());
		/*
		 * !!! If we have more then 1 millis span, conversion to minutes will
		 * give 1 minute always to avoid problems at client code... It is better
		 * then 0 minutes time span. It is a question of rounding - let's say
		 * that we round up when convert from millis to minutes
		 */
		assertEquals(1, t.getSpanMinutes());
		assertEquals(TimeUnit.MILLISECONDS, t.getTimeUnit());
		assertEquals(1, t.getTimeUnitSpan());
	}

	public void testGetSpanMillis() {
		TimeSpan t = new TimeSpan(0);
		assertEquals(0, t.getSpanMillis());
		t = new TimeSpan(10001);
		assertEquals(10001, t.getSpanMillis());
	}

	public void testGetSpanMinutes() {
		TimeSpan t = new TimeSpan(0);
		assertEquals(0, t.getSpanMinutes());
		t = new TimeSpan(1);
		assertEquals(1, t.getSpanMinutes());
		t = new TimeSpan(59999);
		assertEquals(1, t.getSpanMinutes());
		t = new TimeSpan(60000);
		assertEquals(1, t.getSpanMinutes());
		t = new TimeSpan(60001);
		assertEquals(1, t.getSpanMinutes());
		t = new TimeSpan(119999);
		assertEquals(1, t.getSpanMinutes());
		t = new TimeSpan(120000);
		assertEquals(2, t.getSpanMinutes());
	}

	public void testSetMillis() {
		TimeSpan t = new TimeSpan(TimeUnit.HOURS, 100);
		assertEquals(100 * 3600 * 1000, t.getSpanMillis());
		t.setMillis(1);
		assertEquals(1, t.getSpanMillis());
	}

	public void testSetSeconds() {
		TimeSpan t = new TimeSpan(TimeUnit.HOURS, 100);
		assertEquals(100 * 3600 * 1000, t.getSpanMillis());
		t.setSeconds(1);
		assertEquals(1000, t.getSpanMillis());
		assertEquals(1, t.getSpanMinutes());
	}

	public void testSetMinuttes() {
		TimeSpan t = new TimeSpan(TimeUnit.HOURS, 100);
		assertEquals(100 * 3600 * 1000, t.getSpanMillis());
		t.setMinutes(1);
		assertEquals(60000, t.getSpanMillis());
		assertEquals(1, t.getSpanMinutes());
	}

	public void testSetHours() {
		TimeSpan t = new TimeSpan(TimeUnit.HOURS, 100);
		assertEquals(100 * 3600 * 1000, t.getSpanMillis());
		t.setHours(2);
		assertEquals(2 * 60 * 60 * 1000, t.getSpanMillis());
		assertEquals(2 * 60, t.getSpanMinutes());
	}

	public void testToString() {
		TimeSpan t = new TimeSpan(1);
		assertEquals("1 MILLISECONDS", t.toString());
		t = new TimeSpan(6);
		assertEquals("6 MILLISECONDS", t.toString());
		t = new TimeSpan(6000);
		assertEquals("6000 MILLISECONDS", t.toString());
		t.setHours(1);
		assertEquals("1 HOURS", t.toString());
		t.setSeconds(700013);
		assertEquals("700013 SECONDS", t.toString());
		t.setMinutes(84);
		assertEquals("84 MINUTES", t.toString());
	}

	public void testGetTimeUnitTimeUnitSpan() {
		TimeSpan t = new TimeSpan();
		assertEquals(0, t.getTimeUnitSpan());
		assertEquals(TimeUnit.MILLISECONDS, t.getTimeUnit());

		t = new TimeSpan(12);
		assertEquals(12, t.getTimeUnitSpan());
		assertEquals(TimeUnit.MILLISECONDS, t.getTimeUnit());

		t = new TimeSpan(12000);
		assertEquals(12000, t.getTimeUnitSpan());
		assertEquals(TimeUnit.MILLISECONDS, t.getTimeUnit());

		t = new TimeSpan(TimeUnit.DAYS, 12000);
		assertEquals(12000, t.getTimeUnitSpan());
		assertEquals(TimeUnit.DAYS, t.getTimeUnit());
	}

}
