package dk.gov.oiosi.communication.client;

import org.apache.axis2.AxisFault;
import org.apache.axis2.engine.AxisConfiguration;
import org.apache.axis2.engine.AxisConfigurator;

import junit.framework.TestCase;
import dk.gov.oiosi.configuration.ConfigurationHandler;
import dk.gov.oiosi.test.TestConstants;

public class AxisMercuryDispatcherTest extends TestCase {

	/*
	 * Test to reproduce out of memory problem, when transport listener is not
	 * started but shutdown method is called.
	 * 
	 * The test just creates new instance of AxisMercuryDispatcher and then
	 * calls its shutdown() method.
	 * 
	 * In order to get results faster, reduce amount of memory for this class.
	 * 
	 * Results: in case of max memory 64MB (-Xmx64m for HotSpot JVM) and
	 * axisListenerStartupEnabled set to false, the test fails at iteration 179
	 * with OutOfMemory.
	 * 
	 * If axisListenerStartupEnabled set to true, it does not fail even on 1000
	 * iterations.
	 * 
	 * To be able to see details of memory leak, turn on memory dump on
	 * OutOfMemory. For HotSpot JVM it is -XX:+HeapDumpOnOutOfMemoryError
	 */
	public void testShutdownContext() throws Exception {
		final boolean actuallyRunTheTest = false;
		final boolean axisListenerStartupEnabled = false;
		final int iterationCount = 1000;

		if (actuallyRunTheTest) {
			/*
			 * It is not that obvoius, but inside AxisMercuryDispatcher
			 * constructor there is a reference to an instance of
			 * ConfigurationHandler to get http policy...
			 */
			System.out.println("Set configuration file to " + TestConstants.PATH_RASPCONFIGURATION_TEST);
			ConfigurationHandler.setConfigurationFile(TestConstants.PATH_RASPCONFIGURATION_TEST);

			System.out.println("Start iterations with axisListenerStartupEnabled = " + axisListenerStartupEnabled);
			for (int i = 0; i < iterationCount; i++) {
				Runtime rt = Runtime.getRuntime();
				System.out.println("Iteration " + i + " used " + (rt.totalMemory() - rt.freeMemory()) / 1024 / 1024 + " MB of " + rt.maxMemory() / 1024 / 1024 + " MB");
				new AxisMercuryDispatcher("test/conf/client_axis2.xml", "axis2/repository", "test/conf/wss_policy.xml") {
					@Override
					protected boolean isTransportListenerStartupEnabled(AxisConfigurator axisConf) throws AxisFault {
						return axisListenerStartupEnabled;
					}
				};
				AxisMercuryDispatcher.shutdownContext();
			}
		} else {
			System.out.println("AxisMercuryDispatcherTest.testShutdownContext is skipped");
		}
	}

	public void testIsTransportListenerStartupEnabled() throws Exception {
		final Object[] p = new Object[1];
		final AxisConfiguration c = new AxisConfiguration() {
			@Override
			public Object getParameterValue(String name) {
				return p[0];
			}
		};
		final AxisConfiguration[] cl = new AxisConfiguration[] { c };
		final AxisConfigurator conf = new AxisConfigurator() {
			@Override
			public void loadServices() {
			}

			@Override
			public AxisConfiguration getAxisConfiguration() throws AxisFault {
				return cl[0];
			}

			@Override
			public void engageGlobalModules() throws AxisFault {
			}

			@Override
			public void cleanup() {
			}
		};
		final AxisMercuryDispatcher a = new AxisMercuryDispatcher(getName(), getName(), getName()) {
			@Override
			public void init(String s1, String s2, String s3) {
				/*
				 * Do nothing in unit test
				 */
			}
		};
		/*
		 * Default - nothing is defined in axis2.xml
		 */
		assertTrue(a.isTransportListenerStartupEnabled(conf));

		/*
		 * List of possible values for axis2.xml property
		 * "disableTransportListenerStartup", which disable transport listener
		 * startup
		 */
		Object[] disableListenerOptions = new Object[] { "yes", "1", "blabla", "any other string", "true", " false ", Boolean.TRUE, new Integer(1), new Integer(-1), new Integer(100) };
		for (int i = 0; i < disableListenerOptions.length; i++) {
			p[0] = disableListenerOptions[i];
			assertFalse("Expected false for " + p[0], a.isTransportListenerStartupEnabled(conf));
		}

		/*
		 * List of possible values for axis2.xml property
		 * "disableTransportListenerStartup", which enable transport listener startup
		 */
		Object[] enableListenerOptions = new Object[] { null, "0", "no", "false", Boolean.FALSE, new Integer(0) };
		for (int i = 0; i < enableListenerOptions.length; i++) {
			p[0] = enableListenerOptions[i];
			assertTrue("Expected true for " + p[0], a.isTransportListenerStartupEnabled(conf));
		}

		/*
		 * Coverage tests
		 */
		assertTrue(a.isTransportListenerStartupEnabled(null));
		cl[0] = null;
		assertTrue(a.isTransportListenerStartupEnabled(conf));
	}
}
