package dk.gov.oiosi.configuration;

import junit.framework.TestCase;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;

import java.lang.reflect.Method;
import java.net.URL;
import java.util.Collection;

public class ConfigurationHandlerBackwardCompatibilityTest extends TestCase {

    private static final Logger log = Logger.getLogger(ConfigurationHandlerTest.class);

    /*
      * This test checks, what fails if library is just updated without
      * modifications at RaspConfiguration.
      */
    public void testBackwardCompatibility() throws Exception {
        URL resource = this.getClass().getResource("ConfigurationHandlerTest_RaspConfiguration_Old.xml");
        ConfigurationHandler.setConfigurationFile(resource.getFile());
        ConfigurationHandler.clear();

        Method[] declaredMethods = ConfigurationHandler.class.getDeclaredMethods();

        int countErrors = 0;
        int countEmptyResult = 0;
        for (Method method : declaredMethods) {
            if (method.getName().startsWith("get") && method.getParameterTypes().length == 0) {
                try {
                    Object result = method.invoke(ConfigurationHandler.getInstance());
                    boolean isEmpty = false;
                    if (result == null) {
                        countEmptyResult++;
                        isEmpty = true;
                    } else {
                        if (result instanceof Collection<?>) {
                            if (((Collection<?>) result).isEmpty()) {
                                countEmptyResult++;
                                isEmpty = true;
                            }
                        }
                    }
                    System.out.println("ConfigurationHandlerTest.testBackwardCompatibility: " + method.getName() + "\t" + "OK " + (isEmpty ? "empty" : "not empty"));
                } catch (Exception e) {
                    System.out.println("ConfigurationHandlerTest.testBackwardCompatibility: " + method.getName() + "\t" + "KO");
                    log.error("Failed getCacheConfig", e);
                    countErrors++;
                }
            }
        }
        assertEquals(0, countErrors);

        // On old configurations only getStartupTaskList should be empty - rest should be present
        assertEquals(1, countEmptyResult);

        /*
           * Check that cache configuration is valid - initialize all required caches
           *
           * This should not throw any exception
           */
        Logger cfl = Logger.getLogger(CacheFactory.class);
        Level initialLevel = cfl.getLevel();
        try {
            cfl.setLevel(Level.INFO);
            CacheFactory.getInstance();
        } finally {
            cfl.setLevel(initialLevel);
        }
    }
}