package dk.gov.oiosi.configuration;

import dk.gov.oiosi.common.startup.*;
import dk.gov.oiosi.communication.SendPolicy;
import dk.gov.oiosi.exception.MainException;
import dk.gov.oiosi.security.RootCertificateConfig;
import dk.gov.oiosi.test.TestConstants;
import junit.framework.TestCase;
import org.apache.commons.configuration.XMLConfiguration;
import org.apache.commons.configuration.reloading.FileChangedReloadingStrategy;
import org.apache.commons.configuration.tree.xpath.XPathExpressionEngine;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;

import java.net.URL;
import java.util.List;

public class ConfigurationHandlerTest extends TestCase {

    @Override
    public void setUp() throws Exception {
        super.setUp();
        ConfigurationHandler.setConfigurationFile(TestConstants.PATH_RASPCONFIGURATION_TEST);
    }

    public void testCacheConfiguration() {
        ConfigurationHandler.getInstance().getCacheConfig();
        assertTrue("Configuration of cache was not a success", true);
    }

    public void testParseStartupTasks() throws Exception {
        XMLConfiguration config = new XMLConfiguration(TestConstants.PATH_RASPCONFIGURATION_TEST);
        config.setReloadingStrategy(new FileChangedReloadingStrategy());
        config.setExpressionEngine(new XPathExpressionEngine());

        List<IStartupTask> list = ConfigurationHandler.parseStartTaskList(config);
        assertNotNull(list);
        assertEquals("List of tasks: " + list, 3, list.size());

        int i = 0;
        assertEquals(SchematronPreloadStartupTask.class, list.get(i).getClass());
        SchematronPreloadStartupTask t1 = (SchematronPreloadStartupTask) list.get(i);
        assertTrue(!t1.getConfigList().isEmpty());
        assertEquals(4, t1.getConfigList().size());
        assertEquals("initDocumentTypeFriendlyName", t1.getConfigList().get(0).getName());
        assertEquals("Kreditnota", t1.getConfigList().get(1).getValue());

        assertEquals("initDocumentTypeFriendlyName", t1.getConfigList().get(2).getName());
        assertEquals("Faktura v0.7", t1.getConfigList().get(2).getValue());

        assertEquals("prevalidateXmlFolder", t1.getConfigList().get(3).getName());
        assertEquals("Resources/Documents/Examples", t1.getConfigList().get(3).getValue());


        i++;
        assertEquals(DownloadCRLStartupTask.class, list.get(1).getClass());
        DownloadCRLStartupTask t2 = (DownloadCRLStartupTask) list.get(1);
        assertEquals(3, t2.getConfigList().size());

        assertEquals("http://crl.oces.certifikat.dk/oces.crl", t2.getConfigList().get(0).getValue());
        assertEquals("http://crl.systemtest8.trust2408.com/systemtest8.crl", t2.getConfigList().get(1).getValue());
        assertEquals("downloadCrlUrl", t2.getConfigList().get(2).getName());
        assertEquals("http://crl.systemtest7.trust2408.com/systemtest7.crl", t2.getConfigList().get(2).getValue());

        i++;
        assertEquals(StartCacheStateLogScheduledJobStartupTask.class, list.get(i).getClass());

        StartCacheStateLogScheduledJobStartupTask t3 = (StartCacheStateLogScheduledJobStartupTask) list.get(i);
        assertEquals(2, t3.getConfigList().size());

        assertEquals("scheduleJobRunInterval", t3.getConfigList().get(0).getName());
        assertEquals("SECONDS", t3.getConfigList().get(1).getValue());
    }

    /*
      * Unit test to check logs
      */
    public void testExecuteStartupTasks() throws MainException {
        // Use Log4J for configuration of log level
        Logger log1 = Logger.getLogger(ConfigurationHandler.class);
        Level initialLevel1 = log1.getLevel();
        Logger log2 = Logger.getLogger(AbstractStartupTask.LOG_NAME);
        Level initialLevel2 = log2.getLevel();
        try {
            log1.setLevel(Level.INFO);
            log2.setLevel(Level.INFO);
            ConfigurationHandler.getInstance().executeStartupTasks();
//			return;
        } catch (Throwable e) {
            /*
                * TODO DLK: Evaluate necessity to throw exception from
                * executeStartupTasks which should stop system startup in case of
                * serious problem, e.g. invalid root certificate (nothing will work
                * in this case...)
                *
                * See comment to ScheduledExecutionTerminateRuntimeException
                */
            e.printStackTrace();
            fail("No error is expected to happen at this point, it should be safe to call this method, all problems should be just logged, but error happened: " + e.getMessage());
        } finally {
            log1.setLevel(initialLevel1);
            log2.setLevel(initialLevel2);
        }
    }

    public void testInitMailConfigurationNoEmailSection() throws Exception {
        URL resource = this.getClass().getResource("ConfigurationHandlerTest_RaspConfiguration_noEmail.xml");
        ConfigurationHandler.clear();
        ConfigurationHandler.setConfigurationFile(resource.getFile());

        SendPolicy sendPolicy = ConfigurationHandler.getInstance().getSendPolicy();
        assertNull("Email section is absent, so object should be null", sendPolicy.getInboxMailConfiguration());
    }

    public void testInitMailConfigurationWithEmailSection() throws Exception {
        ConfigurationHandler.clear();
        SendPolicy sendPolicy = ConfigurationHandler.getInstance().getSendPolicy();
        assertNotNull("Email section is present, so object should have data", sendPolicy.getInboxMailConfiguration());
    }

    public void testGetRootCertificateLocationCollection() {
        ConfigurationHandler configurationHandler = ConfigurationHandler.getInstance();
        List<RootCertificateConfig> rootCertificateList = configurationHandler.getRootCertificateLocationCollection();

        assertNotNull("Root certificate was not collected", rootCertificateList);
        assertEquals("Wrong count of RootCertificateLocationCollection/RootCertificateLocation in RaspConfiguration.Test.xml", 1, rootCertificateList.size());
    }
}