package dk.gov.oiosi.security.oces;

import dk.gov.oiosi.configuration.CacheFactory;
import dk.gov.oiosi.configuration.ConfigurationHandler;
import dk.gov.oiosi.security.CertificateIssuerSubject;
import dk.gov.oiosi.security.revocation.crl.CrlLookup;
import dk.gov.oiosi.security.revocation.crl.CrlLookupTest;
import dk.gov.oiosi.test.TestConstants;
import junit.framework.TestCase;

import java.security.cert.X509Certificate;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;

public class CertificateUtilTest extends TestCase {

	private static final boolean MAKE_DEVELOPER_MISTAKE = false;

	public void setUp() {
		ConfigurationHandler.setConfigurationFile(TestConstants.PATH_RASPCONFIGURATION_TEST);
	}

	public void testDownloadIssuerCert() throws Exception {
		Level initialLevel = Logger.getLogger(CrlLookup.class).getLevel();
		try {
			Logger.getLogger(CrlLookup.class).setLevel(Level.DEBUG);
			CertificateUtil u = new CertificateUtil();
			X509Certificate cert = CrlLookupTest.getCertificate("server_tdc");
			u.downloadIssuerCert(cert);
			u.downloadIssuerCert(cert);
			if (MAKE_DEVELOPER_MISTAKE) {
				/*
				 * TODO: Review downloadIssuerCert code to avoid eternal loops
				 * if the system is hacked
				 * 
				 * You should never do it like this: adding certificate into
				 * cache with key of this certificate's issuer subject!
				 * 
				 * It will lead to eternal loop in downloadIssuerCert() method
				 * which ends with OutOfMemory
				 */
				CacheFactory.getInstance().getCertificateCache().add(new CertificateIssuerSubject(cert), cert);
				u.downloadIssuerCert(cert);
			}
			CrlLookup cl = new CrlLookup();
			cl.checkCertificate(CrlLookupTest.getCertificate("server_tdc_revoked"));
		} finally {
			Logger.getLogger(CrlLookup.class).setLevel(initialLevel);
		}
	}
}