package dk.gov.oiosi.security.revocation.crl;

import java.net.URL;

/**
 * This program shows, that it is not really correct to use URL object as a key
 * in collections, because its hash code function uses resolved IP address as a
 * part of hash.
 * <p>
 * In OIORASP 1.2.3.HotFix1 at least in the class {@link CrlCache} revocation
 * certificates are stored in Hashtable:
 * <p>
 * private Hashtable<URL, CrlInstance> table
 * <p>
 * This test shows, that it is not really good, because of the server
 * experiences network problems, it will create a new record in this set!
 * <p>
 * Please find example of the program output, generated on Windows 7 OS and Java
 * HotSpot(TM) 64-Bit Server VM (build 20.0-b11, mixed mode), JRE build
 * 1.6.0_25-b06, in file <b>CrlCacheDesignIssueAnalysis_Result.txt</b>
 * 
 * @author Dmitriy Lapko, TrueLink A/S
 */
public class CrlCacheDesignIssueAnalysis {

	private static final String REAL_CRL_ADDRESS = "http://crl.oces.certifikat.dk/oces.crl";
	private static final long NETWORK_AVAILABLE_HASH = -1440447174;
	private static final long NETWORK_UNAVAILABLE_HASH = 940188;

	public static void main(String[] args) throws Exception {
		long start = System.currentTimeMillis();
		URL url = new URL(REAL_CRL_ADDRESS);
		long originalHashCode = url.hashCode();
		System.out.println("URL " + url + ", hashCode " + url.hashCode());
		System.out.println("In normal situation we expect all URL objects, built by the same http address, to have the same hash code.");
		System.out.println("But if the system, where this program runs, experiences network problems, a hash code of URL will be different, because it includes resolved IP address.");
		System.out.println("Let's reproduce this situation.");

		System.out.println("Start to execute 200 attempts to build URL object by the same address and wait 1 second between each search for it in the set.");
		System.out.println("IMPORTANT! In order to check the test, try to turn off your network now and wait some time (~ 30 seconds) until new results appear.");
		System.out.println("Than try to turn on your network again.");

		long previousHashCode = originalHashCode;
		printNetworkStatus(previousHashCode);

		for (int i = 0; i < 100; i++) {
			URL url2 = new URL(REAL_CRL_ADDRESS);
			System.out.print(".");
			long currentHashCode = url2.hashCode();
			if (currentHashCode != previousHashCode) {
				System.out.println();
				System.out.println("**************************************");
				System.out.println("Iteration " + i + ", " + (System.currentTimeMillis() - start) / 1000 + " seconds after program start.");
				System.out.println("URL " + url2 + ", current hashCode = " + currentHashCode);
				System.out.println("Attention! URL object, built by the same URL line, now has another hashCode (" + currentHashCode + ") then before (" + previousHashCode + ")");
				previousHashCode = currentHashCode;
				System.out.println("Now URL object is found in the set!");
				System.out.println("Try to switch your network on/off again.");
				printNetworkStatus(currentHashCode);
			}
			Thread.sleep(1000);
		}
	}

	protected static void printNetworkStatus(long currentHashCode) {
		if (currentHashCode == NETWORK_AVAILABLE_HASH) {
			System.out.println("Now your networks looks ENABLED");
		} else if (currentHashCode == NETWORK_UNAVAILABLE_HASH) {
			System.out.println("Now your networks looks DISABLED");
		}
	}
}
