/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.security.revocation.ocsp;

import dk.gov.oiosi.configuration.ConfigurationHandler;
import dk.gov.oiosi.configuration.ConfigurationHandlerCleaner;
import dk.gov.oiosi.test.TestConstants;
import junit.framework.TestCase;

import java.security.cert.X509Certificate;
import java.util.List;

public class OcspConfigTest extends TestCase {

    OcspConfig ocspConfig;

    @Override
    public void setUp() throws Exception {
        super.setUp();

        ConfigurationHandlerCleaner.clear();
        ConfigurationHandler.setConfigurationFile(TestConstants.PATH_RASPCONFIGURATION_TEST);

        ocspConfig = new OcspConfig();
    }

    /**
     * https://bugs.softwareborsen.dk/show_bug.cgi?id=1277
     * <p/>
     * Verify patch, where multiple RootCertificateLocation elements in RaspConfiguration.xml will be added to final certificate list,
     * and not only the certificates found in the last JKS file.
     *
     * @throws Exception On error...
     */
    public void testGetDefaultOcesRootCertificateCollectionFromStore() throws Exception {
        ConfigurationHandlerCleaner.clear();
        ConfigurationHandler.setConfigurationFile("./test/resources/RaspConfiguration_multipleRootCertificateLocations.xml");

        List<X509Certificate> certList = ocspConfig.getDefaultOcesRootCertificateCollectionFromStore();
        assertNotNull("Returned cert should not be null", certList);
        assertEquals("List count wrong", 4, certList.size());
    }
}