/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.test.service;

import dk.gov.oiosi.communication.OiosiMessage;
import dk.gov.oiosi.communication.service.IServiceContractTransaction;
import dk.gov.oiosi.exception.MainException;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMNamespace;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.context.ServiceContext;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.util.Iterator;
import java.util.Map;

/**
 * Interop test service.
 */
public class OiosiService implements IServiceContractTransaction {

    /**
     * local log.
     */
    private static final Log log = LogFactory.getLog(OiosiService.class);

    /**
     * service context.
     */
    ServiceContext serviceContext;

    /**
     * Init service context.
     *
     * @param sc
     */
    public void init(ServiceContext sc) {
        serviceContext = sc;
    }

    /**
     * invocation counter.
     */
    protected static int invokeCounter = 0;

    /**
     * successfull service invocations.
     */
    protected static int confirmCounter = 0;

    /**
     * service invocations with timeout.
     */
    protected static int timeoutCounter = 0;

    /**
     * service invocation with application faults.
     */
    protected static int faultedCounter = 0;

    /* (non-Javadoc)
     * @see dk.gov.oiosi.communication.service.IServiceContractTransaction#confirmOperation(java.lang.String)
     */
    public void confirmOperation(String messageId) {
        confirmCounter++;
        log.warn("Service - C O N F I R M : total/confirm/timeout/fault = " + invokeCounter + "/" + confirmCounter + "/"
                + timeoutCounter + "/" + faultedCounter + " : " + messageId);
    }

    /* (non-Javadoc)
     * @see dk.gov.oiosi.communication.service.IServiceContractTransaction#timeoutOperation(java.lang.String)
     */
    public void timeoutOperation(String messageId) {
        timeoutCounter++;
        log.warn("Service - T I M E O U T - Operation : total/confirm/timeout/ = " + invokeCounter + "/" + confirmCounter + "/"
                + timeoutCounter + "/" + faultedCounter + " : " + messageId);
    }

    /* (non-Javadoc)
     * @see dk.gov.oiosi.communication.service.IServiceContractTransaction#faultedOperation(java.lang.String)
     */
    public void faultedOperation(String messageId) {
        faultedCounter++;
        log.warn("Service - F A U L T E D - Operation : total/confirm/timeout = " + invokeCounter + "/" + confirmCounter + "/"
                + timeoutCounter + "/" + faultedCounter + " : " + messageId);
    }

    /* (non-Javadoc)
     * @see dk.gov.oiosi.communication.service.IServiceContract#requestRespond(dk.gov.oiosi.communication.OiosiMessage)
     */
    public OiosiMessage requestRespond(OiosiMessage message) throws MainException {
        OMElement document = message.getMessageXml().getOMDocument();
        Map<String, Object> customHeaders = message.getMessageHeaders();
        String messageId = message.getMessageIdentifier();

        MessageContext messageCtx = MessageContext.getCurrentMessageContext();
        log.warn("Service invoked : " + messageCtx.getTo().getAddress());

        if( message.getSignerCertificate() != null ) {
            log.error( "SIGNER : " + message.getSignerCertificate().getSubjectX500Principal() );
        }
        invokeCounter++;
        String msg = "Stat : total/confirm/timeout = " + invokeCounter + "/" + confirmCounter + "/" + timeoutCounter + "/"
                + faultedCounter;

        if (document == null) {
            log.warn("OIOSI Test Service received empty document : " + msg);
        } else {
            log.warn("OIOSI Test Service received DOCUMENT : " + messageId + " :: " + document.getLocalName() + " : " + msg);
        }

        //log.warn("MSG : " + document );
        //TODO: Why are we iterating all these namespaces without using them? For test?
        Iterator<?> iterator = document.getAllDeclaredNamespaces();
        while(iterator.hasNext()) {
            OMNamespace ns = (OMNamespace)iterator.next();
            log.warn("NS : " + ns.getNamespaceURI());
        }

        // log.info( "" + docService );
        if (customHeaders == null || customHeaders.isEmpty()) {
            log.warn(" - service received NO custom headers ");
        } else {
            log.warn(" - service received custom headers ");
            for (String key : customHeaders.keySet()) {
                log.info(" - CustomKey : " + key + " : " + customHeaders.get(key));
            }
        }

        log.warn(" - RETURN NULL AS REPLY");
        return null;
    }

}
