/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.test.unit.communication;

import dk.gov.oiosi.Constants;
import dk.gov.oiosi.addressing.IdentifierCvr;
import dk.gov.oiosi.communication.*;
import dk.gov.oiosi.configuration.ConfigurationHandler;
import dk.gov.oiosi.exception.MainException;
import dk.gov.oiosi.security.oces.OcesX509Certificate;
import dk.gov.oiosi.test.TestConstants;
import dk.gov.oiosi.test.util.TestUtil;
import dk.gov.oiosi.xml.XmlDocument;
import junit.framework.TestCase;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.io.FileInputStream;
import java.security.KeyStore;
import java.security.cert.X509Certificate;

/**
 * Test parameters/exception handling for Request.
 */
public class RequestSetupTest extends TestCase {

	static {
        ConfigurationHandler.setConfigurationFile(TestConstants.PATH_RASPCONFIGURATION_TEST);
    }
	
    protected Credentials getCredentials() throws Exception {
        // define receiver certificate
        KeyStore ks = KeyStore.getInstance("JKS");
        ks.load(new FileInputStream(TestConstants.PATH_KEYSTORE_TEST), TestConstants.PASSWORD_KEYSTORE_TEST.toCharArray());

        X509Certificate cert = (X509Certificate) ks.getCertificate("server");
        OcesX509Certificate ocesCert = new OcesX509Certificate(cert);
        
        X509Certificate cert2 = (X509Certificate) ks.getCertificate("client");
        OcesX509Certificate ocesCert2 = new OcesX509Certificate(cert2);
        return new Credentials(ocesCert2, ocesCert);
    }

    /**
     * local log.
     */
    private static Log log = LogFactory.getLog(RequestSetupTest.class);

    public void missingHeaderIdentifier(String header) {
        try {
            SendPolicy policy = new SendPolicy();
            Credentials credentials = TestUtil.getCredentials();

            Uri uri = new Uri("http://dummy.url/service");
            Request request = new Request(uri, credentials, policy);

            XmlDocument xml = new XmlDocument();
            OiosiMessage message = new OiosiMessage(xml);
            if( ! Constants.RASP_MESSAGE_IDENTIFIER.equals(header)) {
                message.addMessageHeader(Constants.RASP_MESSAGE_IDENTIFIER, "urn:uuid:" + System.nanoTime());
            }
            if( ! Constants.RASP_RECEIVER_PARTY_IDENTIFIER.equals(header)) {
                message.setReceiverPartyIdentifier(new IdentifierCvr("11111111"));
            }
            if( ! Constants.RASP_SENDER_PARTY_IDENTIFIER.equals(header)) {
                message.setSenderPartyIdentifier(new IdentifierCvr("22222222"));
            }
            request.getResponse(message);

            // this should fail
            String errorMsg = "Request should detect missing 'MessageIdentifier - when client policy is set to validate custom headers";
            log.error(errorMsg);
            fail(errorMsg);
        } catch (IllegalArgumentException e) {
            log.info("IllegalArgumentException : " + e, e);
        } catch (MainException e) {
            // for now MainException
            log.info("MainException : " + e);
        } catch (Exception e) {
            log.info("Exception : " + e, e);
        }
    }

    public void testMissingHeaderMessageIdentifier() {
        missingHeaderIdentifier(Constants.RASP_MESSAGE_IDENTIFIER);
    }
    public void testMissingHeaderReceiverPartyIdentifier() {
        missingHeaderIdentifier(Constants.RASP_RECEIVER_PARTY_IDENTIFIER);
    }
    public void testMissingHeaderSenderPartyIdentifier() {
        missingHeaderIdentifier(Constants.RASP_SENDER_PARTY_IDENTIFIER);
    }

    
}
