package dk.gov.oiosi.test.unit.configuration;

import dk.gov.oiosi.common.ValUtil;
import dk.gov.oiosi.communication.configuration.*;
import dk.gov.oiosi.configuration.ConfigurationHandler;
import dk.gov.oiosi.test.TestConstants;
import dk.gov.oiosi.uddi.UddiConfig;
import junit.framework.TestCase;

public class RaspConfigurationTest extends TestCase {

    @Override
    public void setUp() throws Exception {
        super.setUp();
        ConfigurationHandler.setConfigurationFile(TestConstants.PATH_RASPCONFIGURATION_TEST);
    }

    public void testDocumentTypeConfiguration() throws Exception {
        ConfigurationHandler handler = ConfigurationHandler.getInstance();
        DocumentTypeCollectionConfig documentTypeCollectionConfig = handler.getRaspDocumentTypeCollectionConfig();
        DocumentTypeConfig[] docTypes = documentTypeCollectionConfig.getDocumentTypes();
        assertTrue(docTypes.length > 0);
        DocumentTypeConfig docType = docTypes[0];
        docType.getCustomHeaderConfiguration().getXPaths();

        assertEndpointType(docType.getEndpointType());
        assertNotNullOrEmpty(docType.getFriendlyName());
        assertNotNullOrEmpty(docType.getId().getGuid());
        assertPositiveInteger(docType.getIdentifierDiscriminators().getXPathDiscriminatorConfigs().length);
        assertPositiveInteger(docType.getNamespaces().length);
        assertNotNullOrEmpty(docType.getRootName());
        assertNotNullOrEmpty(docType.getRootNamespace());
        assertNotNullOrEmpty(docType.getSchemaPath());
        assertNotNullOrEmpty(docType.getServiceContractTModel());
        assertNotNullOrEmpty(docType.getStylesheetPath());
        assertNotNull(docType.getXsltTransformStylesheetPath());
    }

    public void testUddiConfiguration() throws Exception {
        UddiConfig uc = ConfigurationHandler.getInstance().getUddiConfig();
        assertPositiveInteger(uc.getFallbackTimeoutMinutes());

        // Not used anymore - Removed from unit test
        // assertNotNullOrEmpty(uc.getRegistrationConformanceClaim());
        // assertNotNullOrEmpty(uc.getPublishEndpoint());
        // assertNotNullOrEmpty(uc.getSecurityEndpoint());
    }

    private void assertEndpointType(DocumentEndpointInformation endpointInfo) throws KeyTypeMappingFailedException {
        assertNotNull(endpointInfo);
        assertNotNullOrEmpty(endpointInfo.getReplyAction());
        assertNotNullOrEmpty(endpointInfo.getRequestAction());

        ServiceEndpointFriendlyName friendlyName = endpointInfo.getEndpointFriendlyName();
        assertNotNullOrEmpty(friendlyName.getXPath());
        ServiceEndpointFriendlyName senderFriendlyName = endpointInfo.getSenderFriendlyName();
        assertNotNullOrEmpty(senderFriendlyName.getXPath());

        String keyType = "EndpointKeyType";
        ServiceEndpointKey key = endpointInfo.getKey();
        KeyTypeMappingExpression keyMapping = key.getMappingExpression(keyType);
        assertKeyTypeMappingExpression(keyMapping);

        ServiceEndpointKey senderKey = endpointInfo.getSenderKey();
        KeyTypeMappingExpression senderKeyMapping = senderKey.getMappingExpression(keyType);
        assertKeyTypeMappingExpression(senderKeyMapping);
    }

    private void assertKeyTypeMappingExpression(KeyTypeMappingExpression keyMapping) throws KeyTypeMappingFailedException {
        assertNotNull(keyMapping);
        String cvrMappingName = "CVR";
        KeyTypeMapping mapping = keyMapping.getMapping(cvrMappingName);
        assertNotNull(mapping);
    }

    private void assertNotNullOrEmpty(String s) {
        assertNotNull("String was null", s);
        assertFalse("String was empty", ValUtil.isNullOrEmpty(s));
    }

    private void assertPositiveInteger(int i) {
        assertTrue("Negative integer found [" + i + "] - expected non-negative value", i >= 0);
    }
}