/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.test.unit.security.oces;

import dk.gov.oiosi.common.OutVariable;
import dk.gov.oiosi.configuration.ConfigurationHandler;
import dk.gov.oiosi.security.CertificateSubject;
import dk.gov.oiosi.security.oces.OcesCertificateType;
import dk.gov.oiosi.security.oces.OcesX509Certificate;
import dk.gov.oiosi.test.TestConstants;
import junit.framework.TestCase;

import java.io.FileInputStream;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;

/**
 * test oces certificate types. TODO: find out what to test.
 */
public class OcesX509CertificateTest extends TestCase {

    /**
     * Legal subject DN.
     */
    public final static String SUBJECT_DN_OK = "CN=service.ebx.dk (funktionscertifikat), SERIALNUMBER=CVR:25894375-FID:1200312847621, O=MYSUPPLY ApS // CVR:25894375, C=DK";
    /**
     * Illegal subject DN - contains spaces around '='. CertificateSubject should handle this.
     */
    public final static String SUBJECT_DN_ILLEGAL = "CN = service.ebx.dk (funktionscertifikat), SERIALNUMBER = CVR:25894375-FID:1200312847621, O = MYSUPPLY ApS // CVR:25894375, C = DK";

    @Override
    public void setUp() throws Exception {
        super.setUp();
        ConfigurationHandler.setConfigurationFile(TestConstants.PATH_RASPCONFIGURATION_TEST);
    }

    public void testEmployeeType() throws Exception {
        X509Certificate certificate = loadCertificate(TestConstants.PATH_MOCES1_VALID);
        OcesX509Certificate ocesCertificate = new OcesX509Certificate(certificate);
        assertEquals("Wrong cert. type", OcesCertificateType.OcesEmployee, ocesCertificate.getOcesCertificateType().getType());
    }

    public void testOrganisationType() throws Exception {
        X509Certificate certificate = loadCertificate(TestConstants.PATH_VOCES1_TEST);
        OcesX509Certificate ocesCertificate = new OcesX509Certificate(certificate);
        assertEquals("Wrong type found", OcesCertificateType.OcesOrganisation, ocesCertificate.getOcesCertificateType().getType());
    }

    public void testFunctionType() throws Exception {
        X509Certificate certificate = loadCertificate(TestConstants.PATH_FOCES1_VALID);
        OcesX509Certificate ocesCertificate = new OcesX509Certificate(certificate);
        assertEquals("Wrong type", OcesCertificateType.OcesDevice, ocesCertificate.getOcesCertificateType().getType());
    }

    public void testPersonType() throws Exception {
        X509Certificate certificate = loadCertificate(TestConstants.PATH_CERTIFICATE_PERSON);
        OcesX509Certificate ocesCertificate = new OcesX509Certificate(certificate);
        assertTrue(OcesCertificateType.OcesPersonal == ocesCertificate.getOcesCertificateType().getType());
    }

    /**
     * Test illegal subject DN - contains spaces around '='. CertificateSubject should handle this.
     */
    public void testSubjectIllegal() {
        CertificateSubject subject = new CertificateSubject(SUBJECT_DN_ILLEGAL);
        assertNotNull("Subject not created", subject);
    }

    public void testSubjectOk() {
        CertificateSubject subject = new CertificateSubject(SUBJECT_DN_OK);
        assertNotNull("Subject not created", subject);
    }

    public void testTryGetCvrOk() throws Exception {
        X509Certificate certificate = loadCertificate(TestConstants.PATH_MOCES1_VALID);
        OcesX509Certificate ocesCertificate = new OcesX509Certificate(certificate);

        OutVariable<String> cvrNumberString = new OutVariable<String>();
        boolean success = ocesCertificate.TryGetCvrNumberString(cvrNumberString);
        assertTrue(success);
        assertEquals("30808460", cvrNumberString.getVariable());
    }

    public void testTryGetCvrFail() throws Exception {
        X509Certificate certificate = loadCertificate(TestConstants.PATH_CERTIFICATE_PERSON);
        OcesX509Certificate ocesCertificate = new OcesX509Certificate(certificate);

        OutVariable<String> cvrNumberString = new OutVariable<String>();
        boolean success = ocesCertificate.TryGetCvrNumberString(cvrNumberString);
        assertFalse(success);
    }

    /**
     * util to load certificate.
     *
     * @param file filename
     * @return certificate
     * @throws Exception On error...
     */
    private X509Certificate loadCertificate(String file) throws Exception {
        CertificateFactory factory = CertificateFactory.getInstance("X509");
        return (X509Certificate) factory.generateCertificate(new FileInputStream(file));
    }
}