package dk.gov.oiosi.test.unit.uddi;

import dk.gov.oiosi.addressing.Identifier;
import dk.gov.oiosi.addressing.IdentifierEan;
import dk.gov.oiosi.communication.Uri;
import dk.gov.oiosi.configuration.ConfigurationHandler;
import dk.gov.oiosi.exception.MainException;
import dk.gov.oiosi.test.TestConstants;
import dk.gov.oiosi.test.unit.uddi.AdvancedUddiDummyClient.AdvancedUddiDummyClientConfig;
import dk.gov.oiosi.uddi.*;
import junit.framework.TestCase;

import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

public class RegistryLookupTest extends TestCase {
	private Uri firstRegistryUri;
	private Uri firstFallbackUri;
	private Uri secondRegistryUri;
	private Uri thirdRegistryUri;
    private Uri secondFallbackUri;
	private Uri fourthRegistryUri;
	private Uri fourthFallbackUri1;
	private Uri fourthFallbackUri2;

	private IdentifierEan endpointInNoRegistry = new IdentifierEan("5700000000000");
 	private IdentifierEan endpointInFirstRegistry = new IdentifierEan("5700000000001");
 	private IdentifierEan endpointInSecondRegistry = new IdentifierEan("5700000000002");
	private IdentifierEan endpointInFourthRegistry = new IdentifierEan("5700000000004");

	static {
        ConfigurationHandler.setConfigurationFile(TestConstants.PATH_RASPCONFIGURATION_TEST);
    }
	
	@Override
	public void setUp() {
		try {
			firstRegistryUri = new Uri("http://test1.com");
			firstFallbackUri = new Uri("http://fallback1.com");
			secondRegistryUri = new Uri("http://test2.com");
			thirdRegistryUri = new Uri("http://test3.com");
		    secondFallbackUri = new Uri("http://fallback2.com");
			fourthRegistryUri = new Uri("http://test4.com");
			fourthFallbackUri1 = new Uri("http://fallback41.com");
			fourthFallbackUri2 = new Uri("http://fallback42.com");

			SetUpConfiguration();
			
		} catch (URISyntaxException e) {
			e.printStackTrace();
		}
	}

	 public void test_01_SuccesfulOnFirstLookup() throws MainException  {
     	GetClearDummyConfig();
     	
     	IUddiLookupClient client = RegistryLookupClientFactory.getRegistryLookupClient();
		List<UddiLookupResponse> result = client.lookup(CreateParams(endpointInFirstRegistry));
		assertNotNull(result);
		assertTrue(result.size() > 0);
		assertEquals(firstRegistryUri.toString(), result.get(0).getEndpointAddress().getKeyAsString());
     }

     public void test_02_SuccesfulOnFirstFallback() throws MainException {
		AdvancedUddiDummyClient.AdvancedUddiDummyClientConfig config = GetClearDummyConfig();
		config.getErroneousEndpoints().add(firstRegistryUri);
		IUddiLookupClient client = RegistryLookupClientFactory.getRegistryLookupClient();
		List<UddiLookupResponse> result = client.lookup(CreateParams(endpointInFirstRegistry));
		assertNotNull(result);
		assertTrue(result.size() > 0);
		assertEquals(firstFallbackUri.toString(), result.get(0).getEndpointAddress().getKeyAsString());
     }

    public void test_03_FailureOnFirstRegistry() throws MainException {
 		AdvancedUddiDummyClient.AdvancedUddiDummyClientConfig config = GetClearDummyConfig();
		config.getErroneousEndpoints().add(firstRegistryUri);
		config.getErroneousEndpoints().add(firstFallbackUri);
		IUddiLookupClient client = RegistryLookupClientFactory.getRegistryLookupClient();
		try {
			client.lookup(CreateParams(endpointInFirstRegistry));
		}
		catch (UddiException e) {
			return;
		}
		fail();
     }

 	public void test_04_SuccesfulOnSecondRegistry() throws MainException {
    	AdvancedUddiDummyClient.AdvancedUddiDummyClientConfig config = GetClearDummyConfig();
		List<Identifier> endpointIdentifier = new ArrayList<Identifier>();
		endpointIdentifier.add(endpointInSecondRegistry);
		config.getNonExistingRegistrations().put(firstRegistryUri, endpointIdentifier);
		IUddiLookupClient client = RegistryLookupClientFactory.getRegistryLookupClient();
		List<UddiLookupResponse> result = client.lookup(CreateParams(endpointInSecondRegistry));
		assertNotNull(result);
		assertTrue(result.size() > 0);
		assertEquals(secondRegistryUri.toString(), result.get(0).getEndpointAddress().getKeyAsString());
	}    

	public void test_05_SuccesfulOnLastFallback() throws MainException {
		AdvancedUddiDummyClient.AdvancedUddiDummyClientConfig config = GetClearDummyConfig();

		List<Identifier> endpointIdentifier = new ArrayList<Identifier>();
		endpointIdentifier.add(endpointInFourthRegistry);

		config.getNonExistingRegistrations().put(firstRegistryUri, endpointIdentifier);
		config.getNonExistingRegistrations().put(secondRegistryUri, endpointIdentifier);
		config.getNonExistingRegistrations().put(thirdRegistryUri, endpointIdentifier);
		config.getErroneousEndpoints().add(fourthRegistryUri);
		config.getErroneousEndpoints().add(fourthFallbackUri1);
	
		IUddiLookupClient client = RegistryLookupClientFactory.getRegistryLookupClient();
		List<UddiLookupResponse> result = client.lookup(CreateParams(endpointInFourthRegistry));
		assertNotNull(result);
		assertTrue(result.size() > 0);
		assertEquals(fourthFallbackUri2.toString(), result.get(0).getEndpointAddress().getKeyAsString());
	}
	
	 public void test_06_UnsuccesfulOnAllRegistries() throws MainException {
		AdvancedUddiDummyClient.AdvancedUddiDummyClientConfig config = GetClearDummyConfig();

		List<Identifier> endpointIdentifier = new ArrayList<Identifier>();
		endpointIdentifier.add(endpointInNoRegistry);

		
		config.getNonExistingRegistrations().put(firstRegistryUri, endpointIdentifier);
		config.getNonExistingRegistrations().put(secondRegistryUri, endpointIdentifier);
		config.getNonExistingRegistrations().put(thirdRegistryUri, endpointIdentifier);
		config.getNonExistingRegistrations().put(fourthRegistryUri, endpointIdentifier);

		IUddiLookupClient client = RegistryLookupClientFactory.getRegistryLookupClient();
		List<UddiLookupResponse> result = client.lookup(CreateParams(endpointInNoRegistry));
		assertNotNull(result);
		assertTrue(result.size() == 0);
	 }

     private LookupParameters CreateParams(IdentifierEan ean) throws MainException {
    	 List<EndpointAddressTypeCode> endpointTypes = new LinkedList<EndpointAddressTypeCode>();
         endpointTypes.add(EndpointAddressTypeCode.http);
    	 return new LookupParameters(
	         ean,
	         new UddiGuidId("uddi:b138dc71-d301-42d1-8c2e-2c3a26faf56a", true),
	         endpointTypes);
     }

     private AdvancedUddiDummyClient.AdvancedUddiDummyClientConfig GetClearDummyConfig() {
         // Clears the dummy so that all calls return a result
         AdvancedUddiDummyClient.AdvancedUddiDummyClientConfig dummyConfig = AdvancedUddiDummyClientConfig.getInstance();
         //dummyConfig.setErroneousEndpoints(new ArrayList<Uri>());
         //dummyConfig.setNonExistingRegistrations(new HashMap<Uri, List<IIdentifier>>());
         dummyConfig.getNonExistingRegistrations().clear();
         dummyConfig.getErroneousEndpoints().clear();
         return dummyConfig;
     }

     private void SetUpConfiguration() throws URISyntaxException{
         // Configures fallback sequence
    	 ConfigurationHandler handler = ConfigurationHandler.getInstance();
         UddiConfig uddiConfig = handler.getUddiConfig(); 
         uddiConfig.setFallbackTimeoutMinutes(1);
         LookupRegistryFallbackConfig lookupRegistryFallbackConfig = new LookupRegistryFallbackConfig();
         uddiConfig.setLookupRegistryFallbackConfig(lookupRegistryFallbackConfig);
         
         ArrayList<Registry> registries = new ArrayList<Registry>(); 
         
         List<Uri> firstRegistryList = new ArrayList<Uri>();
         firstRegistryList.add(firstRegistryUri);
         firstRegistryList.add(firstFallbackUri);
         registries.add(new Registry(firstRegistryList));
         
         List<Uri> secondRegistryList = new ArrayList<Uri>();
         secondRegistryList.add(secondRegistryUri);
         secondRegistryList.add(secondFallbackUri);
         registries.add(new Registry(secondRegistryList));
         
         List<Uri> thirdRegistryList = new ArrayList<Uri>();
         thirdRegistryList.add(thirdRegistryUri);
         registries.add(new Registry(thirdRegistryList));
         
         List<Uri> fourthRegistryList = new ArrayList<Uri>();
         fourthRegistryList.add(fourthRegistryUri);
         fourthRegistryList.add(fourthFallbackUri1);
         fourthRegistryList.add(fourthFallbackUri2);
         registries.add(new Registry(fourthRegistryList));

         lookupRegistryFallbackConfig.setRegistries(registries);
         // Configures factories for UDDI lookup clients used
         
         RegistryLookupClientFactoryConfig registryLookupClientFactoryConfig = handler.getRegistryLookupClientFactoryConfig();
         registryLookupClientFactoryConfig.setImplementationNamespaceClass("dk.gov.oiosi.uddi.RegistryLookupClient");
         registryLookupClientFactoryConfig.setImplementationAssembly("RegistryLookupClient).Assembly.FullName");

         UddiLookupClientFactoryConfig uddiLookupClientFactoryConfig = handler.getUddiLookupClientFactoryConfig();
         uddiLookupClientFactoryConfig.setImplementationNamespaceClass("dk.gov.oiosi.test.unit.uddi.AdvancedUddiDummyClient");
         uddiLookupClientFactoryConfig.setImplementationAssembly("AdvancedUddiDummyClient).Assembly.FullName");
     }
}