package dk.gov.oiosi.test.unit.uddi;

import dk.gov.oiosi.communication.Uri;
import dk.gov.oiosi.configuration.ConfigurationHandler;
import dk.gov.oiosi.test.TestConstants;
import dk.gov.oiosi.uddi.LookupRegistryFallbackConfig;
import dk.gov.oiosi.uddi.Registry;
import dk.gov.oiosi.uddi.UddiConfig;
import junit.framework.TestCase;

import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

/**
 * Test cases that tests the configuration for the UDDI lookup
 */
public class UddiConfigTest extends TestCase {

    private String _registry1Endpoint1 = "http://test.com";
    private String _registry1Endpoint2 = "http://fallback.com";
    private String _registry2Endpoint1 = "http://test2.com";
    private String _registry2Endpoint2 = "http://fallback2.com";

    private int _fallbackTimeout = 5;

    static {
        ConfigurationHandler.setConfigurationFile(TestConstants.PATH_RASPCONFIGURATION_TEST);
    }


    /**
     * Tests that the values are correctly inserted into the configuration element.
     *
     * @throws URISyntaxException
     */
    public void test_InsertIntoConfig() throws URISyntaxException {
        ConfigurationHandler handler = ConfigurationHandler.getInstance();

        UddiConfig config = handler.getUddiConfig();
        config.setFallbackTimeoutMinutes(_fallbackTimeout);

        LookupRegistryFallbackConfig lookupRegistryFallbackConfig = new LookupRegistryFallbackConfig();
        ArrayList<Registry> registries = new ArrayList<Registry>();
        Registry registry1 = new Registry();
        List<Uri> endpoints1 = new ArrayList<Uri>();
        endpoints1.add(new Uri(_registry1Endpoint1));
        endpoints1.add(new Uri(_registry1Endpoint2));
        registry1.setEndpoints(endpoints1);
        registries.add(registry1);
        Registry registry2 = new Registry();
        List<Uri> endpoints2 = new ArrayList<Uri>();
        endpoints2.add(new Uri(_registry2Endpoint1));
        endpoints2.add(new Uri(_registry2Endpoint2));
        registry2.setEndpoints(endpoints2);
        registries.add(registry2);
        lookupRegistryFallbackConfig.setRegistries(registries);
        config.setLookupRegistryFallbackConfig(lookupRegistryFallbackConfig);
    }

    /**
     * Reads from the configuration and tests that it is possible.
     *
     * @throws Exception
     */
    public void test_GetFromConfig() throws Exception {
        ConfigurationHandler handler = ConfigurationHandler.getInstance();
        UddiConfig config = handler.getUddiConfig();
        assertEquals(_fallbackTimeout, config.getFallbackTimeoutMinutes());

        LookupRegistryFallbackConfig lookupRegistryFallbackConfig = config.getLookupRegistryFallbackConfig();
        assertNotNull(lookupRegistryFallbackConfig);
        Collection<Registry> registries = lookupRegistryFallbackConfig.getRegistries();
        assertEquals(2, registries.size());
        int index1 = 0;
        for (Registry registry : registries) {
            int index2 = 0;
            Collection<Uri> endpoints = registry.getEndpoints();
            for (Uri endpoint : endpoints) {
                if (index1 == 0 && index2 == 0) {
                    assertEquals(_registry1Endpoint1, endpoint.getAbsoluteUri());
                } else if (index1 == 0 && index2 == 1) {
                    assertEquals(_registry1Endpoint2, endpoint.getAbsoluteUri());
                } else if (index1 == 1 && index2 == 0) {
                    assertEquals(_registry2Endpoint1, endpoint.getAbsoluteUri());
                } else if (index1 == 1 && index2 == 1) {
                    assertEquals(_registry2Endpoint2, endpoint.getAbsoluteUri());
                } else {
                    throw new Exception("The indexes are out of bounds " + index1 + ":" + index2);
                }
                index2++;
            }
            index1++;
        }
    }
}