package dk.gov.oiosi.test.unit.xml.documenttype;

import java.io.File;

import junit.framework.TestCase;
import dk.gov.oiosi.communication.configuration.DocumentTypeConfig;
import dk.gov.oiosi.configuration.ConfigurationHandler;
import dk.gov.oiosi.exception.ArgumentNullException;
import dk.gov.oiosi.test.TestConstants;
import dk.gov.oiosi.xml.XmlDocument;
import dk.gov.oiosi.xml.documenttype.DocumentTypeConfigSearcher;
import dk.gov.oiosi.xml.documenttype.NoDocumentTypeFoundFromXmlDocumentException;

/**
 * test document type config search.
 */
public class DocumentTypeConfigSearcherTest extends TestCase {

    private static DocumentTypeConfigSearcher searcher;
    private static final String friendlyName = "No Namespace";

    @Override
    public void setUp() throws Exception {
        super.setUp();
        ConfigurationHandler.setConfigurationFile(TestConstants.PATH_RASPCONFIGURATION_TEST);
        searcher = new DocumentTypeConfigSearcher();
    }

    /**
     * test init searcher.
     *
     * @throws ConfigurationException
     */
    /*public void testDocumentTypeConfigSearcher() throws ConfigurationException {
         searcher = new DocumentTypeConfigSearcher();
     }*/

    /**
     * Searches for the application response 2.01 document type
     *
     * @throws Exception On error...
     */
    public void testSearchForApplicationResponse201() throws Exception {
        assertFindDocument("Applikationsmeddelse", TestConstants.PATH_APPLICATIONRESPONSE_201);
    }

    public void testSearchForApplicationResponse202() throws Exception {
        assertFindDocument("Applikationsmeddelse", TestConstants.PATH_APPLICATIONRESPONSE202_XML);
    }

    public void testSearchForCatalogue() throws Exception {
        assertFindDocument("Katalog", TestConstants.PATH_CATALOGUE);
    }

    public void testSearchForCatalogueDeletion() throws Exception {
        assertFindDocument("Sletning af katalog",
                TestConstants.PATH_CATALOGUEDELETION);
    }

    public void testSearchForCatalogueItemSpecificationUpdate() throws Exception {
        assertFindDocument("Opdatering af katalogelement", TestConstants.PATH_CATALOGUEITEMSPECIFICATIONUPDATE);
    }

    public void testSearchForCataloguePricingUpdate() throws Exception {
        assertFindDocument("Opdatering af katalogpriser",
                TestConstants.PATH_CATALOGUEPRICINGUPDATE);
    }

    public void testSearchForCatalogueRequest() throws Exception {
        assertFindDocument("Katalogforespørgsel",
                TestConstants.PATH_CATALOGUEREQUEST);
    }

    public void testSearchForCreditNote() throws Exception {
        assertFindDocument("Kreditnota", TestConstants.PATH_CREDITNOTE);
    }

    public void testSearchForInvoice() throws Exception {
        assertFindDocument("Faktura", TestConstants.PATH_INVOICE);
    }

    public void testSearchForOrder() throws Exception {
        assertFindDocument("Ordre", TestConstants.PATH_ORDER);
    }

    public void testSearchForOrderCancellation() throws Exception {
        assertFindDocument("Ordreannulering",
                TestConstants.PATH_ORDERCANCELLATION);
    }

    public void testSearchForOrderChange() throws Exception {
        assertFindDocument("Ordreændring", TestConstants.PATH_ORDERCHANGE);
    }

    public void testSearchForOrderResponse() throws Exception {
        assertFindDocument("Ordrebekræftelse",
                TestConstants.PATH_ORDERRESPONSE);
    }

    public void testSearchForOrderResponseSimple() throws Exception {
        assertFindDocument("Simpel ordrebekræftelse",
                TestConstants.PATH_ORDERRESPONSESIMPLE);
    }

    public void testSearchForReminder() throws Exception {
        assertFindDocument("Rykker", TestConstants.PATH_REMINDER);
    }

    public void testSearchForStatement() throws Exception {
        assertFindDocument("KontoUdtog", TestConstants.PATH_STATEMENT);
    }

    public void testSearchForInvoice07() throws Exception {
        assertFindDocument("Faktura v0.7", TestConstants.PATH_INVOICE_PIE);
    }

    public void testSearchForNoNamespace() throws Exception {
        try {
            assertFindDocument(friendlyName, TestConstants.PATH_NONAMESPACE_XML);
            fail("Exception should have been thrown");
        } catch (NoDocumentTypeFoundFromXmlDocumentException e) {
            // OK
        }
    }

    public void testSearchForUnknownType() throws Exception {
        try {
            assertFindDocument(null, TestConstants.PATH_UNKNOWNTYPE_XML);
            fail("Exception should have been thrown");
        } catch (NoDocumentTypeFoundFromXmlDocumentException e) {
            // OK
        }
    }

    public void testNullArgument() throws Exception {
        try {
            searcher.findUniqueDocumentType(null);
            fail("Exception should have been thrown!");
        } catch (ArgumentNullException e) {
            // OK
        }
    }

    private DocumentTypeConfig searchForDocument(String path) throws Exception {
        XmlDocument document = new XmlDocument();
        document.load(path);
        return searcher.findUniqueDocumentType(document);
    }

    private void assertFindDocument(String friendlyName, String documentPath) throws Exception {
        File file = new File(documentPath);
        assertTrue("Test file not found on path: " + documentPath, file.exists());
        DocumentTypeConfig documentType = searchForDocument(documentPath);
        assertEquals(friendlyName, documentType.getFriendlyName());
    }
}