package dk.gov.oiosi.test.unit.xml.schema;

import dk.gov.oiosi.TestUtil;
import dk.gov.oiosi.communication.configuration.DocumentTypeConfig;
import dk.gov.oiosi.configuration.ConfigurationHandler;
import dk.gov.oiosi.test.TestConstants;
import dk.gov.oiosi.xml.XmlDocument;
import dk.gov.oiosi.xml.XmlSchema;
import dk.gov.oiosi.xml.documenttype.DocumentTypeConfigSearcher;
import dk.gov.oiosi.xml.schema.SchemaValidationFailedException;
import dk.gov.oiosi.xml.schema.SchemaValidator;
import dk.gov.oiosi.xml.schema.UnexpectedNamespaceException;
import junit.framework.TestCase;

import java.io.File;

/**
 * Perform schema validation tests on all oioubl documents.
 */
public class SchemaValidationTest extends TestCase {

    private static DocumentTypeConfigSearcher docSearcher;

    @Override
    public void setUp() throws Exception {
        super.setUp();
        ConfigurationHandler.setConfigurationFile(TestConstants.PATH_RASPCONFIGURATION_TEST);

        if (docSearcher == null) {
            docSearcher = new DocumentTypeConfigSearcher();
        }
    }

    public void testApplicationResponseValidation() throws Exception {
        schemaValidateDocument(TestConstants.PATH_APPLICATIONRESPONSE_201);
    }

    public void testCatalogueValidation() throws Exception {
        schemaValidateDocument(TestConstants.PATH_CATALOGUE);
    }

    public void testCatalogueDeletionValidation() throws Exception {
        schemaValidateDocument(TestConstants.PATH_CATALOGUEDELETION);
    }

    public void testCatalogueItemSpecificationUpdateValidation() throws Exception {
        schemaValidateDocument(TestConstants.PATH_CATALOGUEITEMSPECIFICATIONUPDATE);
    }

    public void testCataloguePricingUpdateValidation() throws Exception {
        schemaValidateDocument(TestConstants.PATH_CATALOGUEPRICINGUPDATE);
    }

    public void testCatalogueRequestValidation() throws Exception {
        schemaValidateDocument(TestConstants.PATH_CATALOGUEREQUEST);
    }

    public void testCreditNoteValidation() throws Exception {
        schemaValidateDocument(TestConstants.PATH_CREDITNOTE);
    }

    public void testInvoiceValidation() throws Exception {
        schemaValidateDocument(TestConstants.PATH_INVOICE);
    }

    public void testOrderValidation() throws Exception {
        schemaValidateDocument(TestConstants.PATH_ORDER);
    }

    public void testOrderCancellationValidation() throws Exception {
        schemaValidateDocument(TestConstants.PATH_ORDERCANCELLATION);
    }

    public void testOrderChangeValidation() throws Exception {
        schemaValidateDocument(TestConstants.PATH_ORDERCHANGE);
    }

    public void testOrderResponseValidation() throws Exception {
        schemaValidateDocument(TestConstants.PATH_ORDERRESPONSE);
    }

    public void testOrderResponseSimpleValidation() throws Exception {
        schemaValidateDocument(TestConstants.PATH_ORDERRESPONSESIMPLE);
    }

    public void testReminderValidation() throws Exception {
        schemaValidateDocument(TestConstants.PATH_REMINDER);
    }

    public void testStatementValidation() throws Exception {
        schemaValidateDocument(TestConstants.PATH_STATEMENT);
    }

    public void testUtilityStatementValidation() throws Exception {
        schemaValidateDocument(TestConstants.PATH_UTILITYSTATEMENT_XML);
    }

    public void testInvoice07Validation() throws Exception {
        schemaValidateDocument(TestConstants.PATH_INVOICE_PIE);
    }

    public void testCreditNote07Validation() throws Exception {
        schemaValidateDocument(TestConstants.PATH_CREDITNOTE07_XML);
    }

    public void testInvoiceWrongNamespaceValidation() throws Exception {
        try {
            String xmlPath = TestConstants.PATH_INVOICE_WRONG_NAME_SPACE;
            String schemaPath = TestConstants.PATH_INVOICE_XSD;
            schemaValidateDocument(xmlPath, schemaPath);
            fail("Exception should have been thrown...");
        } catch (SchemaValidationFailedException e) {
            assertTrue(true);
        } catch (UnexpectedNamespaceException e) {
            assertTrue(true);
        }
    }

    public void testInvoiceWrongElementValidation() throws Exception {
        try {
            String xmlPath = TestConstants.PATH_INVOICE_WRONG_ELEMENT;
            String schemaPath = TestConstants.PATH_INVOICE_XSD;
            schemaValidateDocument(xmlPath, schemaPath);
            fail("Exception should have been thrown...");
        } catch (SchemaValidationFailedException e) {
            assertTrue(true);
        }
    }

    /**
     * Verify that it's possible to schema validate all the example files located in "Resources\Documents\Examples".
     */
    public void testSchemaValidateAllTestFiles() {
        for (File file : TestUtil.getOkExampleDocumentPaths()) {
            try {
                schemaValidateDocument(file.getAbsolutePath());
            } catch (Exception e) {
                fail("Unable to Schema validate document [" + TestConstants.PATH_EXAMPLES_OK + file.getName() + "]: " + e.getMessage());
            }
        }
    }

    // Helper methods.

    private void schemaValidateDocument(String xmlDocumentPath) throws Exception {
        SchemaValidator validator = new SchemaValidator();

        XmlDocument document = new XmlDocument();
        document.load(xmlDocumentPath);

        DocumentTypeConfig documentType = docSearcher.findUniqueDocumentType(document);
        String xmlSchemaPath = documentType.getSchemaPath();

        XmlSchema schema = new XmlSchema(xmlSchemaPath);
        validator.schemaValidateXmlDocument(document, schema);
    }

    private void schemaValidateDocument(String xmlDocumentPath, String xmlSchemaPath) throws Exception {
        SchemaValidator validator = new SchemaValidator();

        XmlDocument document = new XmlDocument();
        document.load(xmlDocumentPath);

        XmlSchema schema = new XmlSchema(xmlSchemaPath);
        validator.schemaValidateXmlDocument(document, schema);
    }
}