package dk.gov.oiosi.test.unit.xml.schematron;

import dk.gov.oiosi.common.cache.LeastFrequentlyUsedQuantityCache;
import dk.gov.oiosi.communication.configuration.DocumentTypeCollectionConfig;
import dk.gov.oiosi.communication.configuration.DocumentTypeConfig;
import dk.gov.oiosi.configuration.ConfigurationHandler;
import dk.gov.oiosi.configuration.ConfigurationHandlerCleaner;
import dk.gov.oiosi.test.TestConstants;
import dk.gov.oiosi.xml.XmlDocument;
import dk.gov.oiosi.xml.documenttype.DocumentTypeConfigSearcher;
import dk.gov.oiosi.xml.schematron.SchematronStore;
import dk.gov.oiosi.xml.schematron.SchematronStoreFactory;
import dk.gov.oiosi.xml.schematron.SchematronValidationConfig;
import junit.framework.TestCase;

import javax.xml.transform.Transformer;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

public class SchematronStoreTest extends TestCase {

    static {
        ConfigurationHandler.setConfigurationFile(TestConstants.PATH_RASPCONFIGURATION_TEST);
    }

    public void test_01_GetOnce() throws Exception {
        XmlDocument document = new XmlDocument();
        document.load(TestConstants.PATH_INVOICE);
        DocumentTypeConfigSearcher searcher = new DocumentTypeConfigSearcher();
        DocumentTypeConfig documentTypeConfig = searcher.findUniqueDocumentType(document);
        SchematronStore store = SchematronStoreFactory.getSchematronStore();
        Transformer transform = store.getCompiledSchematron(documentTypeConfig.getSchematronValidationConfig().getSchematronDocumentPath());
        assertNotNull(transform);
    }

    public void test_02_GetTwice() throws Exception {
        XmlDocument document = new XmlDocument();
        document.load(TestConstants.PATH_INVOICE);
        DocumentTypeConfigSearcher searcher = new DocumentTypeConfigSearcher();
        DocumentTypeConfig documentTypeConfig = searcher.findUniqueDocumentType(document);
        SchematronStore store = SchematronStoreFactory.getSchematronStore();
        Transformer transform1 = store.getCompiledSchematron(documentTypeConfig.getSchematronValidationConfig().getSchematronDocumentPath());
        assertNotNull(transform1);
        Transformer transform2 = store.getCompiledSchematron(documentTypeConfig.getSchematronValidationConfig().getSchematronDocumentPath());
        assertNotNull(transform2);
    }

    public void test_03_GetTenTimes() throws Exception {
        XmlDocument document = new XmlDocument();
        document.load(TestConstants.PATH_INVOICE);
        DocumentTypeConfigSearcher searcher = new DocumentTypeConfigSearcher();
        DocumentTypeConfig documentTypeConfig = searcher.findUniqueDocumentType(document);
        SchematronStore store = SchematronStoreFactory.getSchematronStore();
        Transformer transform1 = store.getCompiledSchematron(documentTypeConfig.getSchematronValidationConfig().getSchematronDocumentPath());
        assertNotNull(transform1);
        for (int i = 0; i < 9; i++) {
            Transformer transform2 = store.getCompiledSchematron(documentTypeConfig.getSchematronValidationConfig().getSchematronDocumentPath());
            assertNotNull(transform2);
        }
    }

    public void testGetCompiledSchematron() throws Exception {
        /*
           * This test shows, that even as SchematronStore is optimized to store
           * not more then 2 transnformers at the same time, it will not give
           * significiant performance boost if we cache more - they are loaded
           * pretty fast even from the scratch.
           *
           * But first loading takes much more time then rest, so it is reasonable
           * to load most often used schematron during the system start.
           */

        final boolean showPerDocumentTypeDetailLevel = false;
        boolean cacheIsOneLessThanUniqueSchematronPathes = true;

        ConfigurationHandlerCleaner.clear();
        Map<String, String> configList = ConfigurationHandler.getInstance().getCacheConfig().getSchematronCache().getConfigurationList();

        DocumentTypeCollectionConfig documentTypeCollectionConfig = ConfigurationHandler.getInstance().getRaspDocumentTypeCollectionConfig();
        DocumentTypeConfig[] documentTypes = documentTypeCollectionConfig.getDocumentTypes();
        Set<String> uniqueSchematronPathSet = new HashSet<String>();
        for (DocumentTypeConfig dtc : documentTypes) {
            uniqueSchematronPathSet.add(dtc.getSchematronValidationConfig().getSchematronDocumentPath());
        }

        int maxCacheSize = uniqueSchematronPathSet.size();

        System.out.println("Number of unique schematron pathes: " + maxCacheSize);

        if (cacheIsOneLessThanUniqueSchematronPathes) {
            maxCacheSize = maxCacheSize - 1;
        }
        configList.put(LeastFrequentlyUsedQuantityCache.CONFIG_MAX_SIZE, String.valueOf(maxCacheSize));
        System.out.println("Set max schematron cache size to " + maxCacheSize);

        long totalStart = System.currentTimeMillis();
        for (int i1 = 0; i1 < 3; i1++) {
            long startCycle = System.currentTimeMillis();
            for (int i = 0; i < documentTypes.length; i++) {
                for (int j = 0; j < 3; j++) {
                    long start = System.currentTimeMillis();
                    DocumentTypeConfig dtc = documentTypes[i];

                    SchematronValidationConfig schematronValidationConfig = dtc.getSchematronValidationConfig();

                    /*
                          * Most important line in the test, which actually does
                          * tested job
                          */
                    Transformer transformer = SchematronStoreFactory.getSchematronStore().getCompiledSchematron(schematronValidationConfig.getSchematronDocumentPath());
                    assertNotNull(transformer);

                    if (showPerDocumentTypeDetailLevel) {
                        long duration = System.currentTimeMillis() - start;
                        System.out.println("Cycle " + i1 + ") " + dtc.getFriendlyName() + "\t" + duration);
                    }
                }
            }
            System.out.println("Cycle " + i1 + " total\t" + (System.currentTimeMillis() - startCycle));
        }
        System.out.println("Total\t" + (System.currentTimeMillis() - totalStart));
    }
}