/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.test.util;

import dk.gov.oiosi.Constants;
import dk.gov.oiosi.addressing.IdentifierCvr;
import dk.gov.oiosi.communication.Credentials;
import dk.gov.oiosi.communication.OiosiMessage;
import dk.gov.oiosi.security.oces.OcesX509Certificate;
import dk.gov.oiosi.test.TestConstants;
import dk.gov.oiosi.xml.XmlDocument;
import org.apache.axiom.om.OMAbstractFactory;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMXMLParserWrapper;
import org.apache.axiom.om.impl.llom.factory.OMXMLBuilderFactory;

import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamReader;
import java.io.ByteArrayInputStream;
import java.io.FileInputStream;
import java.security.KeyStore;
import java.security.cert.X509Certificate;

/**
 * Constants for testing.
 */
public class TestUtil {

    public static final String DEFAULT_KEY_LABEL = "server";

    public static X509Certificate getCertificate() throws Exception {
        // define receiver certificate
        KeyStore ks = KeyStore.getInstance("JKS");
        ks.load(new FileInputStream(TestConstants.PATH_KEYSTORE_TEST), TestConstants.PASSWORD_KEYSTORE_TEST.toCharArray());

        return (X509Certificate) ks.getCertificate(DEFAULT_KEY_LABEL);
    }

    public static Credentials getCredentials() throws Exception {
        // define receiver certificate
        KeyStore ks = KeyStore.getInstance("JKS");
        ks.load(new FileInputStream(TestConstants.PATH_KEYSTORE_TEST), TestConstants.PASSWORD_KEYSTORE_TEST.toCharArray());

        X509Certificate cert2 = (X509Certificate) ks.getCertificate("client");
        OcesX509Certificate ocesCert2 = new OcesX509Certificate(cert2);

        X509Certificate cert = (X509Certificate) ks.getCertificate(DEFAULT_KEY_LABEL);
        OcesX509Certificate ocesCert = new OcesX509Certificate(cert);
        return new Credentials(ocesCert2, ocesCert);
    }

    /**
     * test documnet.
     */
    private static final String NOTIFICATIONXML = "<Notification xmlns=\"http://rep.oio.dk/oiosi/xml/schemas/2006/10/01/\"><SenderID>18827737</SenderID><DestinationID>5793635524530</DestinationID><DateSent>2006-08-09</DateSent><Purpose>ResourceUpdate</Purpose><Destination>Resource Database</Destination><NotificationText>XXX YYY ZZZ : 2007-03-08 14:42:20</NotificationText><Description>Catalogue item no. 83376 has been updated from the AT to the DT model. Please update the written documentation.</Description></Notification>";

    /**
     * action for sending 2.01 invoice.
     */
    private static final String INVOICE_ACTION = "http://rep.oio.dk/oiosi.ehandel.gov.dk/xml/schemas/2007/09/01/Invoice201Interface/SubmitInvoiceRequest";

    /**
     * INTEROP_ACTION ...
     */
    private static final String INTEROP_ACTION = "http://rep.oio.dk/oiosi.ehandel.gov.dk/xml/schemas/2007/09/01/InteropRequest";

    /**
     * Init message to send.
     *
     * @return OiosiMessage
     * @throws Exception On error...
     */
    public static OiosiMessage createMessage(boolean addDefaultHeaders) throws Exception {
        return createMessage(null, addDefaultHeaders);
    }

    /**
     * Init message to send.
     *
     * @param file              file to send
     * @param addDefaultHeaders Add default Headers?
     * @return OiosiMessage
     * @throws Exception On error...
     */
    public static OiosiMessage createMessage(String file, boolean addDefaultHeaders) throws Exception {
        XmlDocument document;
        String action = null;
        if (file == null) {
            document = createTestDocument();
            action = INTEROP_ACTION;
        } else {
            document = new XmlDocument();
            document.load(file);
            action = INVOICE_ACTION;
        }
        OiosiMessage message = new OiosiMessage(document);
        message.setRequestAction(action);
        // message.setRequestAction( APP_MSG_ACTION );

        if (addDefaultHeaders) {
            message.addMessageHeader(Constants.RASP_MESSAGE_IDENTIFIER, "urn:uuid:" + System.nanoTime());
            message.setReceiverPartyIdentifier(new IdentifierCvr("11111111"));
            message.setSenderPartyIdentifier(new IdentifierCvr("22222222"));
        }
        return message;
    }

    /**
     * Init test document.
     *
     * @return XmlDocument.
     * @throws Exception On error...
     */
    public static XmlDocument createTestDocument() throws Exception {

        XMLStreamReader parser = XMLInputFactory.newInstance().createXMLStreamReader(
                new ByteArrayInputStream(NOTIFICATIONXML.getBytes()));
        OMXMLParserWrapper builder = OMXMLBuilderFactory.createStAXOMBuilder(OMAbstractFactory.getOMFactory(), parser);
        OMElement doc = builder.getDocumentElement();
        doc.build();

        return new XmlDocument(doc);
    }
}