/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and 
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.uddi;

import dk.gov.oiosi.communication.Uri;
import dk.gov.oiosi.configuration.ConfigurationHandler;
import dk.gov.oiosi.configuration.ConfigurationHandlerCleaner;
import dk.gov.oiosi.test.TestConstants;
import junit.framework.TestCase;

public class UddiFallbackClientTest extends TestCase {

    UddiFallbackClient uddiFallbackClient;

    @Override
    public void setUp() throws Exception {
        super.setUp();

        ConfigurationHandlerCleaner.clear();
        ConfigurationHandler.setConfigurationFile(TestConstants.PATH_RASPCONFIGURATION_PRODUCTION);
        uddiFallbackClient = new UddiFallbackClient(null);
    }

    public void testGetCachedUddiLookupClient() throws Exception {
        assertEquals("Cache should be empty from start", 0, uddiFallbackClient.getUddiLookupClientCacheSize());

        Uri uri1 = new Uri("http://www.nemhandel.dk");
        uddiFallbackClient.getCachedUddiLookupClient(uri1);
        assertEquals("Expected cache count wrong", 1, uddiFallbackClient.getUddiLookupClientCacheSize());

        Uri uri2 = new Uri("http://www.nemhandel2.dk");
        uddiFallbackClient.getCachedUddiLookupClient(uri2);
        assertEquals("Expected cache count wrong", 2, uddiFallbackClient.getUddiLookupClientCacheSize());

        // Use same URL as in Uri1 - that should not make a new cache entry!
        Uri uri3 = new Uri("http://www.nemhandel.dk");
        uddiFallbackClient.getCachedUddiLookupClient(uri3);
        assertEquals("Expected cache count wrong", 2, uddiFallbackClient.getUddiLookupClientCacheSize());
    }
}